from typing import Generic, Literal, Optional, Type, Union, cast, overload

from weaviate.collections.classes.filters import (
    Filter,
)
from weaviate.collections.classes.grpc import PROPERTIES, REFERENCES, MetadataQuery
from weaviate.collections.classes.internal import (
    CrossReferences,
    MetadataSingleObjectReturn,
    ObjectSingleReturn,
    QuerySingleReturn,
    ReturnProperties,
    ReturnReferences,
    _QueryOptions,
)
from weaviate.collections.classes.types import (
    Properties,
    References,
    TProperties,
    TReferences,
)
from weaviate.collections.queries.base_executor import _BaseExecutor
from weaviate.connect import executor
from weaviate.connect.v4 import ConnectionType
from weaviate.proto.v1 import search_get_pb2
from weaviate.types import INCLUDE_VECTOR, UUID


class _FetchObjectByIDQueryExecutor(
    Generic[ConnectionType, Properties, References], _BaseExecutor[ConnectionType]
):
    @overload
    def fetch_object_by_id(
        self,
        uuid: UUID,
        include_vector: INCLUDE_VECTOR = False,
        *,
        return_properties: Union[PROPERTIES, bool, None] = None,
        return_references: Literal[None] = None,
    ) -> executor.Result[ObjectSingleReturn[Properties, References]]: ...

    @overload
    def fetch_object_by_id(
        self,
        uuid: UUID,
        include_vector: INCLUDE_VECTOR = False,
        *,
        return_properties: Union[PROPERTIES, bool, None] = None,
        return_references: REFERENCES,
    ) -> executor.Result[ObjectSingleReturn[Properties, CrossReferences]]: ...

    @overload
    def fetch_object_by_id(
        self,
        uuid: UUID,
        include_vector: INCLUDE_VECTOR = False,
        *,
        return_properties: Union[PROPERTIES, bool, None] = None,
        return_references: Type[TReferences],
    ) -> executor.Result[ObjectSingleReturn[Properties, TReferences]]: ...

    @overload
    def fetch_object_by_id(
        self,
        uuid: UUID,
        include_vector: INCLUDE_VECTOR = False,
        *,
        return_properties: Type[TProperties],
        return_references: Literal[None] = None,
    ) -> executor.Result[ObjectSingleReturn[TProperties, References]]: ...

    @overload
    def fetch_object_by_id(
        self,
        uuid: UUID,
        include_vector: INCLUDE_VECTOR = False,
        *,
        return_properties: Type[TProperties],
        return_references: REFERENCES,
    ) -> executor.Result[ObjectSingleReturn[TProperties, CrossReferences]]: ...

    @overload
    def fetch_object_by_id(
        self,
        uuid: UUID,
        include_vector: INCLUDE_VECTOR = False,
        *,
        return_properties: Type[TProperties],
        return_references: Type[TReferences],
    ) -> executor.Result[ObjectSingleReturn[TProperties, TReferences]]: ...

    @overload
    def fetch_object_by_id(
        self,
        uuid: UUID,
        include_vector: INCLUDE_VECTOR = False,
        *,
        return_properties: Optional[ReturnProperties[TProperties]] = None,
        return_references: Optional[ReturnReferences[TReferences]] = None,
    ) -> executor.Result[QuerySingleReturn[Properties, References, TProperties, TReferences]]: ...

    def fetch_object_by_id(
        self,
        uuid: UUID,
        include_vector: INCLUDE_VECTOR = False,
        *,
        return_properties: Optional[ReturnProperties[TProperties]] = None,
        return_references: Optional[ReturnReferences[TReferences]] = None,
    ):
        """Retrieve an object from the server by its UUID.

        Args:
            uuid: The UUID of the object to retrieve, REQUIRED.
            include_vector: Whether to include the vector in the returned object.
            return_properties: The properties to return for each object.
            return_references: The references to return for each object.

        NOTE:
            - If `return_properties` is not provided then all properties are returned except for blob properties.
            - If `return_metadata` is not provided then no metadata is provided. Use MetadataQuery.full() to retrieve all metadata.
            - If `return_references` is not provided then no references are provided.

        Raises:
            weaviate.exceptions.WeaviateGRPCQueryError: If the network connection to Weaviate fails.
            weaviate.exceptions.WeaviateInsertInvalidPropertyError: If a property is invalid. I.e., has name `id` or `vector`, which are reserved.
        """
        return_metadata = MetadataQuery(
            creation_time=True, last_update_time=True, is_consistent=True
        )

        def resp(
            res: search_get_pb2.SearchReply,
        ) -> QuerySingleReturn[Properties, References, TProperties, TReferences]:
            objects = self._result_to_query_return(
                res,
                _QueryOptions.from_input(
                    return_metadata,
                    return_properties,
                    include_vector,
                    self._references,
                    return_references,
                ),
            )
            if len(objects.objects) == 0:
                return None

            obj = objects.objects[0]
            assert obj.metadata is not None
            assert obj.metadata.creation_time is not None
            assert obj.metadata.last_update_time is not None

            return cast(
                QuerySingleReturn[Properties, References, TProperties, TReferences],
                ObjectSingleReturn(
                    uuid=obj.uuid,
                    vector=obj.vector,
                    properties=obj.properties,
                    metadata=MetadataSingleObjectReturn(
                        creation_time=obj.metadata.creation_time,
                        last_update_time=obj.metadata.last_update_time,
                        is_consistent=obj.metadata.is_consistent,
                    ),
                    references=obj.references,
                    collection=obj.collection,
                ),
            )

        request = self._query.get(
            limit=1,
            filters=Filter.by_id().equal(uuid),
            return_metadata=self._parse_return_metadata(return_metadata, include_vector),
            return_properties=self._parse_return_properties(return_properties),
            return_references=self._parse_return_references(return_references),
        )
        return executor.execute(
            response_callback=resp, method=self._connection.grpc_search, request=request
        )



from typing import Generic, Literal, Optional, Type, Union, overload

from weaviate.collections.classes.filters import _Filters
from weaviate.collections.classes.grpc import (
    METADATA,
    PROPERTIES,
    REFERENCES,
    GroupBy,
    NearVectorInputType,
    Rerank,
    TargetVectorJoinType,
)
from weaviate.collections.classes.internal import (
    CrossReferences,
    GroupByReturn,
    QueryReturn,
    QuerySearchReturnType,
    ReturnProperties,
    ReturnReferences,
)
from weaviate.collections.classes.types import Properties, References, TProperties, TReferences
from weaviate.connect.v4 import ConnectionAsync
from weaviate.types import INCLUDE_VECTOR, NUMBER

from .executor import _NearVectorQueryExecutor

class _NearVectorQueryAsync(
    Generic[Properties, References],
    _NearVectorQueryExecutor[ConnectionAsync, Properties, References],
):
    @overload
    async def near_vector(
        self,
        near_vector: NearVectorInputType,
        *,
        certainty: Optional[NUMBER] = None,
        distance: Optional[NUMBER] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        auto_limit: Optional[int] = None,
        filters: Optional[_Filters] = None,
        group_by: Literal[None] = None,
        rerank: Optional[Rerank] = None,
        target_vector: Optional[TargetVectorJoinType] = None,
        include_vector: INCLUDE_VECTOR = False,
        return_metadata: Optional[METADATA] = None,
        return_properties: Union[PROPERTIES, bool, None] = None,
        return_references: Literal[None] = None,
    ) -> QueryReturn[Properties, References]: ...
    @overload
    async def near_vector(
        self,
        near_vector: NearVectorInputType,
        *,
        certainty: Optional[NUMBER] = None,
        distance: Optional[NUMBER] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        auto_limit: Optional[int] = None,
        filters: Optional[_Filters] = None,
        group_by: Literal[None] = None,
        rerank: Optional[Rerank] = None,
        target_vector: Optional[TargetVectorJoinType] = None,
        include_vector: INCLUDE_VECTOR = False,
        return_metadata: Optional[METADATA] = None,
        return_properties: Union[PROPERTIES, bool, None] = None,
        return_references: REFERENCES,
    ) -> QueryReturn[Properties, CrossReferences]: ...
    @overload
    async def near_vector(
        self,
        near_vector: NearVectorInputType,
        *,
        certainty: Optional[NUMBER] = None,
        distance: Optional[NUMBER] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        auto_limit: Optional[int] = None,
        filters: Optional[_Filters] = None,
        group_by: Literal[None] = None,
        rerank: Optional[Rerank] = None,
        target_vector: Optional[TargetVectorJoinType] = None,
        include_vector: INCLUDE_VECTOR = False,
        return_metadata: Optional[METADATA] = None,
        return_properties: Union[PROPERTIES, bool, None] = None,
        return_references: Type[TReferences],
    ) -> QueryReturn[Properties, TReferences]: ...
    @overload
    async def near_vector(
        self,
        near_vector: NearVectorInputType,
        *,
        certainty: Optional[NUMBER] = None,
        distance: Optional[NUMBER] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        auto_limit: Optional[int] = None,
        filters: Optional[_Filters] = None,
        group_by: Literal[None] = None,
        rerank: Optional[Rerank] = None,
        target_vector: Optional[TargetVectorJoinType] = None,
        include_vector: INCLUDE_VECTOR = False,
        return_metadata: Optional[METADATA] = None,
        return_properties: Type[TProperties],
        return_references: Literal[None] = None,
    ) -> QueryReturn[TProperties, References]: ...
    @overload
    async def near_vector(
        self,
        near_vector: NearVectorInputType,
        *,
        certainty: Optional[NUMBER] = None,
        distance: Optional[NUMBER] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        auto_limit: Optional[int] = None,
        filters: Optional[_Filters] = None,
        group_by: Literal[None] = None,
        rerank: Optional[Rerank] = None,
        target_vector: Optional[TargetVectorJoinType] = None,
        include_vector: INCLUDE_VECTOR = False,
        return_metadata: Optional[METADATA] = None,
        return_properties: Type[TProperties],
        return_references: REFERENCES,
    ) -> QueryReturn[TProperties, CrossReferences]: ...
    @overload
    async def near_vector(
        self,
        near_vector: NearVectorInputType,
        *,
        certainty: Optional[NUMBER] = None,
        distance: Optional[NUMBER] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        auto_limit: Optional[int] = None,
        filters: Optional[_Filters] = None,
        group_by: Literal[None] = None,
        rerank: Optional[Rerank] = None,
        target_vector: Optional[TargetVectorJoinType] = None,
        include_vector: INCLUDE_VECTOR = False,
        return_metadata: Optional[METADATA] = None,
        return_properties: Type[TProperties],
        return_references: Type[TReferences],
    ) -> QueryReturn[TProperties, TReferences]: ...
    @overload
    async def near_vector(
        self,
        near_vector: NearVectorInputType,
        *,
        certainty: Optional[NUMBER] = None,
        distance: Optional[NUMBER] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        auto_limit: Optional[int] = None,
        filters: Optional[_Filters] = None,
        group_by: GroupBy,
        rerank: Optional[Rerank] = None,
        target_vector: Optional[TargetVectorJoinType] = None,
        include_vector: INCLUDE_VECTOR = False,
        return_metadata: Optional[METADATA] = None,
        return_properties: Union[PROPERTIES, bool, None] = None,
        return_references: Literal[None] = None,
    ) -> GroupByReturn[Properties, References]: ...
    @overload
    async def near_vector(
        self,
        near_vector: NearVectorInputType,
        *,
        certainty: Optional[NUMBER] = None,
        distance: Optional[NUMBER] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        auto_limit: Optional[int] = None,
        filters: Optional[_Filters] = None,
        group_by: GroupBy,
        rerank: Optional[Rerank] = None,
        target_vector: Optional[TargetVectorJoinType] = None,
        include_vector: INCLUDE_VECTOR = False,
        return_metadata: Optional[METADATA] = None,
        return_properties: Union[PROPERTIES, bool, None] = None,
        return_references: REFERENCES,
    ) -> GroupByReturn[Properties, CrossReferences]: ...
    @overload
    async def near_vector(
        self,
        near_vector: NearVectorInputType,
        *,
        certainty: Optional[NUMBER] = None,
        distance: Optional[NUMBER] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        auto_limit: Optional[int] = None,
        filters: Optional[_Filters] = None,
        group_by: GroupBy,
        rerank: Optional[Rerank] = None,
        target_vector: Optional[TargetVectorJoinType] = None,
        include_vector: INCLUDE_VECTOR = False,
        return_metadata: Optional[METADATA] = None,
        return_properties: Union[PROPERTIES, bool, None] = None,
        return_references: Type[TReferences],
    ) -> GroupByReturn[Properties, TReferences]: ...
    @overload
    async def near_vector(
        self,
        near_vector: NearVectorInputType,
        *,
        certainty: Optional[NUMBER] = None,
        distance: Optional[NUMBER] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        auto_limit: Optional[int] = None,
        filters: Optional[_Filters] = None,
        group_by: GroupBy,
        rerank: Optional[Rerank] = None,
        target_vector: Optional[TargetVectorJoinType] = None,
        include_vector: INCLUDE_VECTOR = False,
        return_metadata: Optional[METADATA] = None,
        return_properties: Type[TProperties],
        return_references: Literal[None] = None,
    ) -> GroupByReturn[TProperties, References]: ...
    @overload
    async def near_vector(
        self,
        near_vector: NearVectorInputType,
        *,
        certainty: Optional[NUMBER] = None,
        distance: Optional[NUMBER] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        auto_limit: Optional[int] = None,
        filters: Optional[_Filters] = None,
        group_by: GroupBy,
        rerank: Optional[Rerank] = None,
        target_vector: Optional[TargetVectorJoinType] = None,
        include_vector: INCLUDE_VECTOR = False,
        return_metadata: Optional[METADATA] = None,
        return_properties: Type[TProperties],
        return_references: REFERENCES,
    ) -> GroupByReturn[TProperties, CrossReferences]: ...
    @overload
    async def near_vector(
        self,
        near_vector: NearVectorInputType,
        *,
        certainty: Optional[NUMBER] = None,
        distance: Optional[NUMBER] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        auto_limit: Optional[int] = None,
        filters: Optional[_Filters] = None,
        group_by: GroupBy,
        rerank: Optional[Rerank] = None,
        target_vector: Optional[TargetVectorJoinType] = None,
        include_vector: INCLUDE_VECTOR = False,
        return_metadata: Optional[METADATA] = None,
        return_properties: Type[TProperties],
        return_references: Type[TReferences],
    ) -> GroupByReturn[TProperties, TReferences]: ...
    @overload
    async def near_vector(
        self,
        near_vector: NearVectorInputType,
        *,
        certainty: Optional[NUMBER] = None,
        distance: Optional[NUMBER] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        auto_limit: Optional[int] = None,
        filters: Optional[_Filters] = None,
        group_by: Optional[GroupBy] = None,
        rerank: Optional[Rerank] = None,
        target_vector: Optional[TargetVectorJoinType] = None,
        include_vector: INCLUDE_VECTOR = False,
        return_metadata: Optional[METADATA] = None,
        return_properties: Optional[ReturnProperties[TProperties]] = None,
        return_references: Optional[ReturnReferences[TReferences]] = None,
    ) -> QuerySearchReturnType[Properties, References, TProperties, TReferences]: ...



import datetime
import struct
import time
import uuid as uuid_package
from typing import Any, Dict, List, Mapping, Optional, Sequence, Union, cast

from google.protobuf.struct_pb2 import Struct

from weaviate.collections.classes.batch import (
    BatchObject,
    BatchObjectReturn,
    ErrorObject,
    _BatchObject,
)
from weaviate.collections.classes.config import ConsistencyLevel
from weaviate.collections.classes.internal import ReferenceInputs, ReferenceToMulti
from weaviate.collections.classes.types import GeoCoordinate, PhoneNumber
from weaviate.collections.grpc.shared import _BaseGRPC, _is_1d_vector, _Pack
from weaviate.connect import executor
from weaviate.connect.v4 import Connection
from weaviate.exceptions import (
    WeaviateInsertInvalidPropertyError,
    WeaviateInsertManyAllFailedError,
    WeaviateInvalidInputError,
)
from weaviate.proto.v1 import base_pb2, batch_pb2
from weaviate.types import VECTORS
from weaviate.util import _datetime_to_string, _ServerVersion


class _BatchGRPC(_BaseGRPC):
    """This class is used to insert multiple objects into Weaviate using the gRPC API.

    It is used within the `_Data` and `_Batch` classes hence the necessary generalities
    and abstractions so as not to couple to strongly to either use-case.
    """

    def __init__(
        self,
        weaviate_version: _ServerVersion,
        consistency_level: Optional[ConsistencyLevel],
    ):
        super().__init__(weaviate_version, consistency_level, False)

    def __single_vec(self, vectors: Optional[VECTORS]) -> Optional[bytes]:
        if not _is_1d_vector(vectors):
            return None
        return _Pack.single(vectors)

    def __multi_vec(self, vectors: Optional[VECTORS]) -> Optional[List[base_pb2.Vectors]]:
        if vectors is None or _is_1d_vector(vectors):
            return None
        # pylance fails to type narrow TypeGuard in _is_1d_vector properly
        vectors = cast(Mapping[str, Union[Sequence[float], Sequence[Sequence[float]]]], vectors)
        return [
            base_pb2.Vectors(name=name, vector_bytes=packing.bytes_, type=packing.type_)
            for name, vec_or_vecs in vectors.items()
            if (packing := _Pack.parse_single_or_multi_vec(vec_or_vecs))
        ]

    def __grpc_objects(self, objects: List[_BatchObject]) -> List[batch_pb2.BatchObject]:
        return [
            batch_pb2.BatchObject(
                collection=obj.collection,
                uuid=(str(obj.uuid) if obj.uuid is not None else str(uuid_package.uuid4())),
                properties=(
                    self.__translate_properties_from_python_to_grpc(
                        obj.properties,
                        obj.references if obj.references is not None else {},
                    )
                    if obj.properties is not None
                    else None
                ),
                tenant=obj.tenant,
                vector_bytes=self.__single_vec(obj.vector),
                vectors=self.__multi_vec(obj.vector),
            )
            for obj in objects
        ]

    def objects(
        self,
        connection: Connection,
        *,
        objects: List[_BatchObject],
        timeout: Union[int, float],
        max_retries: float,
    ) -> executor.Result[BatchObjectReturn]:
        """Insert multiple objects into Weaviate through the gRPC API.

        Args:
            connection: The connection to the Weaviate instance.
            objects: A list of `WeaviateObject` containing the data of the objects to be inserted. The class name must be
                provided for each object, and the UUID is optional. If no UUID is provided, one will be generated for each object.
                The UUIDs of the inserted objects will be returned in the `uuids` attribute of the returned `_BatchReturn` object.
                The UUIDs of the objects that failed to be inserted will be returned in the `errors` attribute of the returned `_BatchReturn` object.
            timeout: The timeout in seconds for the request.
            max_retries: The maximum number of retries in case of a failure.
        """
        weaviate_objs = self.__grpc_objects(objects)
        start = time.time()

        def resp(errors: Dict[int, str]) -> BatchObjectReturn:
            if len(errors) == len(weaviate_objs):
                # Escape sequence (backslash) not allowed in expression portion of f-string prior to Python 3.12: pylance
                raise WeaviateInsertManyAllFailedError(
                    "Here is the set of all errors: {}".format(
                        "\n".join(err for err in set(errors.values()))
                    )
                )

            elapsed_time = time.time() - start
            all_responses: List[Union[uuid_package.UUID, ErrorObject]] = cast(
                List[Union[uuid_package.UUID, ErrorObject]],
                list(range(len(weaviate_objs))),
            )
            return_success: Dict[int, uuid_package.UUID] = {}
            return_errors: Dict[int, ErrorObject] = {}
            for idx, weav_obj in enumerate(weaviate_objs):
                obj = objects[idx]
                if idx in errors:
                    error = ErrorObject(
                        errors[idx],
                        BatchObject._from_internal(obj),
                        original_uuid=obj.uuid,
                    )
                    return_errors[obj.index] = error
                    all_responses[idx] = error
                else:
                    success = uuid_package.UUID(weav_obj.uuid)
                    return_success[obj.index] = success
                    all_responses[idx] = success

            return BatchObjectReturn(
                uuids=return_success,
                errors=return_errors,
                has_errors=len(errors) > 0,
                _all_responses=all_responses,
                elapsed_seconds=elapsed_time,
            )

        request = batch_pb2.BatchObjectsRequest(
            objects=weaviate_objs,
            consistency_level=self._consistency_level,
        )
        return executor.execute(
            response_callback=resp,
            method=connection.grpc_batch_objects,
            request=request,
            timeout=timeout,
            max_retries=max_retries,
        )

    def __translate_properties_from_python_to_grpc(
        self, data: Dict[str, Any], refs: ReferenceInputs
    ) -> batch_pb2.BatchObject.Properties:
        _validate_props(data)

        multi_target: List[batch_pb2.BatchObject.MultiTargetRefProps] = []
        single_target: List[batch_pb2.BatchObject.SingleTargetRefProps] = []
        non_ref_properties: Struct = Struct()
        bool_arrays: List[base_pb2.BooleanArrayProperties] = []
        text_arrays: List[base_pb2.TextArrayProperties] = []
        int_arrays: List[base_pb2.IntArrayProperties] = []
        float_arrays: List[base_pb2.NumberArrayProperties] = []
        object_properties: List[base_pb2.ObjectProperties] = []
        object_array_properties: List[base_pb2.ObjectArrayProperties] = []
        empty_lists: List[str] = []

        for key, ref in refs.items():
            if isinstance(ref, ReferenceToMulti):
                multi_target.append(
                    batch_pb2.BatchObject.MultiTargetRefProps(
                        uuids=ref.uuids_str,
                        target_collection=ref.target_collection,
                        prop_name=key,
                    )
                )
            elif isinstance(ref, str) or isinstance(ref, uuid_package.UUID):
                single_target.append(
                    batch_pb2.BatchObject.SingleTargetRefProps(uuids=[str(ref)], prop_name=key)
                )
            elif isinstance(ref, list):
                single_target.append(
                    batch_pb2.BatchObject.SingleTargetRefProps(
                        uuids=[str(v) for v in ref], prop_name=key
                    )
                )
            else:
                raise WeaviateInvalidInputError(f"Invalid reference: {ref}")

        for key, entry in data.items():
            if isinstance(entry, dict):
                parsed = self.__translate_properties_from_python_to_grpc(entry, {})
                object_properties.append(
                    base_pb2.ObjectProperties(
                        prop_name=key,
                        value=base_pb2.ObjectPropertiesValue(
                            non_ref_properties=parsed.non_ref_properties,
                            int_array_properties=parsed.int_array_properties,
                            text_array_properties=parsed.text_array_properties,
                            number_array_properties=parsed.number_array_properties,
                            boolean_array_properties=parsed.boolean_array_properties,
                            object_properties=parsed.object_properties,
                            object_array_properties=parsed.object_array_properties,
                            empty_list_props=parsed.empty_list_props,
                        ),
                    )
                )
            elif isinstance(entry, list) and len(entry) == 0:
                empty_lists.append(key)
            elif isinstance(entry, list) and isinstance(entry[0], dict):
                entry = cast(List[Dict[str, Any]], entry)
                object_array_properties.append(
                    base_pb2.ObjectArrayProperties(
                        values=[
                            base_pb2.ObjectPropertiesValue(
                                non_ref_properties=parsed.non_ref_properties,
                                int_array_properties=parsed.int_array_properties,
                                text_array_properties=parsed.text_array_properties,
                                number_array_properties=parsed.number_array_properties,
                                boolean_array_properties=parsed.boolean_array_properties,
                                object_properties=parsed.object_properties,
                                object_array_properties=parsed.object_array_properties,
                                empty_list_props=parsed.empty_list_props,
                            )
                            for v in entry
                            if (parsed := self.__translate_properties_from_python_to_grpc(v, {}))
                        ],
                        prop_name=key,
                    )
                )
            elif isinstance(entry, list) and isinstance(entry[0], bool):
                bool_arrays.append(base_pb2.BooleanArrayProperties(prop_name=key, values=entry))
            elif isinstance(entry, list) and isinstance(entry[0], str):
                text_arrays.append(base_pb2.TextArrayProperties(prop_name=key, values=entry))
            elif isinstance(entry, list) and isinstance(entry[0], datetime.datetime):
                text_arrays.append(
                    base_pb2.TextArrayProperties(
                        prop_name=key, values=[_datetime_to_string(x) for x in entry]
                    )
                )
            elif isinstance(entry, list) and isinstance(entry[0], uuid_package.UUID):
                text_arrays.append(
                    base_pb2.TextArrayProperties(prop_name=key, values=[str(x) for x in entry])
                )
            elif isinstance(entry, list) and isinstance(entry[0], int):
                int_arrays.append(base_pb2.IntArrayProperties(prop_name=key, values=entry))
            elif isinstance(entry, list) and isinstance(entry[0], float):
                values_bytes = struct.pack("{}d".format(len(entry)), *entry)
                float_arrays.append(
                    base_pb2.NumberArrayProperties(prop_name=key, values_bytes=values_bytes)
                )
            elif isinstance(entry, GeoCoordinate):
                non_ref_properties.update({key: entry._to_dict()})
            elif isinstance(entry, PhoneNumber):
                non_ref_properties.update({key: entry._to_dict()})
            else:
                non_ref_properties.update({key: _serialize_primitive(entry)})

        return batch_pb2.BatchObject.Properties(
            non_ref_properties=non_ref_properties,
            multi_target_ref_props=multi_target,
            single_target_ref_props=single_target,
            text_array_properties=text_arrays,
            number_array_properties=float_arrays,
            int_array_properties=int_arrays,
            boolean_array_properties=bool_arrays,
            object_properties=object_properties,
            object_array_properties=object_array_properties,
            empty_list_props=empty_lists,
        )


def _validate_props(props: Dict[str, Any]) -> None:
    if "id" in props or "vector" in props:
        raise WeaviateInsertInvalidPropertyError(props)


def _serialize_primitive(value: Any) -> Any:
    if isinstance(value, uuid_package.UUID):
        return str(value)
    if isinstance(value, datetime.datetime):
        return _datetime_to_string(value)
    if isinstance(value, list):
        return [_serialize_primitive(val) for val in value]

    return value



from typing import Literal, Optional, overload

from weaviate.cluster.models import (
    ReplicateOperationWithHistory,
    ReplicateOperationWithoutHistory,
)
from weaviate.connect.v4 import ConnectionAsync
from weaviate.types import UUID

from .executor import _ReplicateExecutor

class _ReplicateAsync(_ReplicateExecutor[ConnectionAsync]):
    @overload
    async def get(
        self, *, uuid: UUID, include_history: Literal[False] = False
    ) -> Optional[ReplicateOperationWithoutHistory]: ...
    @overload
    async def get(
        self, *, uuid: UUID, include_history: Literal[True]
    ) -> Optional[ReplicateOperationWithHistory]: ...
    async def list_all(self) -> list[ReplicateOperationWithHistory]: ...
    @overload
    async def query(
        self,
        *,
        collection: Optional[str] = None,
        shard: Optional[str] = None,
        target_node: Optional[str] = None,
        include_history: Literal[True],
    ) -> list[ReplicateOperationWithHistory]: ...
    @overload
    async def query(
        self,
        *,
        collection: Optional[str] = None,
        shard: Optional[str] = None,
        target_node: Optional[str] = None,
        include_history: Literal[False] = False,
    ) -> list[ReplicateOperationWithoutHistory]: ...
    async def cancel(self, *, uuid: UUID) -> None: ...
    async def delete(self, *, uuid: UUID) -> None: ...
    async def delete_all(self) -> None: ...



from weaviate.collections.backups.executor import _CollectionBackupExecutor
from weaviate.connect import executor
from weaviate.connect.v4 import ConnectionAsync


@executor.wrap("async")
class _CollectionBackupAsync(_CollectionBackupExecutor[ConnectionAsync]):
    pass



import datetime

import pytest

import weaviate
import weaviate.classes as wvc
from weaviate.collections.classes.filters import (
    Filter,
    _FilterAnd,
    _FilterNot,
    _FilterOr,
    _Operator,
)
from weaviate.proto.v1 import base_pb2


def test_empty_input_contains_any() -> None:
    with pytest.raises(weaviate.exceptions.WeaviateInvalidInputError):
        wvc.query.Filter.by_id().contains_any([])
    with pytest.raises(weaviate.exceptions.WeaviateInvalidInputError):
        wvc.query.Filter.by_property("test").contains_any([])


def test_empty_input_contains_all() -> None:
    with pytest.raises(weaviate.exceptions.WeaviateInvalidInputError):
        wvc.query.Filter.by_property("test").contains_all([])


def test_filter_lists() -> None:
    f1 = wvc.query.Filter.by_property("test").equal("test")
    f2 = wvc.query.Filter.by_creation_time().greater_or_equal(datetime.datetime.now())

    and_list = wvc.query.Filter.all_of([f1, f2])
    and_direct = f1 & f2
    assert isinstance(and_list, _FilterAnd)
    assert isinstance(and_direct, _FilterAnd)
    assert and_list.filters == and_direct.filters

    or_list = wvc.query.Filter.any_of([f1, f2])
    or_direct = f1 | f2
    assert isinstance(or_list, _FilterOr)
    assert isinstance(or_direct, _FilterOr)
    assert or_list.filters == or_direct.filters


def test_filter_lists_one_entry() -> None:
    f1 = wvc.query.Filter.by_property("test").equal("test")

    and_list = wvc.query.Filter.all_of([f1])
    assert and_list == f1

    or_list = wvc.query.Filter.any_of([f1])
    assert or_list == f1


def test_filter_lists_empty() -> None:
    with pytest.raises(weaviate.exceptions.WeaviateInvalidInputError):
        wvc.query.Filter.all_of([])

    with pytest.raises(weaviate.exceptions.WeaviateInvalidInputError):
        wvc.query.Filter.any_of([])


def test_filter_bitwise_and_assignment() -> None:
    f1 = wvc.query.Filter.by_property("test").equal("test")
    f2 = wvc.query.Filter.by_creation_time().greater_or_equal(datetime.datetime.now())
    f3 = wvc.query.Filter.by_update_time().less_or_equal(datetime.datetime.now())

    f4 = f1 & f2
    f4 &= f3
    and_direct = wvc.query.Filter.all_of([f1, f2]) & f3
    assert isinstance(f4, _FilterAnd)
    assert isinstance(and_direct, _FilterAnd)

    assert isinstance(f4.filters[0], _FilterAnd)
    assert isinstance(and_direct.filters[0], _FilterAnd)
    assert f4.filters[0].filters == and_direct.filters[0].filters
    assert f4.filters[1] == f3


def test_filter_bitwise_or_assignment() -> None:
    f1 = wvc.query.Filter.by_property("test").equal("test")
    f2 = wvc.query.Filter.by_creation_time().greater_or_equal(datetime.datetime.now())
    f3 = wvc.query.Filter.by_update_time().less_or_equal(datetime.datetime.now())

    f4 = f1 | f2
    f4 |= f3
    or_direct = wvc.query.Filter.any_of([f1, f2]) | f3
    assert isinstance(f4, _FilterOr)
    assert isinstance(or_direct, _FilterOr)

    assert isinstance(f4.filters[0], _FilterOr)
    assert isinstance(or_direct.filters[0], _FilterOr)
    assert f4.filters[0].filters == or_direct.filters[0].filters
    assert f4.filters[1] == f3


def test_filter_bitwise_invert_assignment() -> None:
    f1 = wvc.query.Filter.by_property("test").equal("test")
    not_f1 = wvc.query.Filter.not_(f1)

    invert_f1 = ~f1

    assert isinstance(invert_f1, _FilterNot)
    assert isinstance(not_f1, _FilterNot)
    assert len(invert_f1.filters) == 1
    assert invert_f1.filters == not_f1.filters


def test_auto_capitalize_first_letter_by_ref_multi_target() -> None:
    result = Filter.by_ref_multi_target(link_on="ref1", target_collection="test")
    target_collection_stored = result._FilterByRef__target.target_collection
    assert target_collection_stored == "Test"


@pytest.mark.parametrize(
    "operator,want",
    [
        (_Operator.EQUAL, base_pb2.Filters.OPERATOR_EQUAL),
        (_Operator.NOT_EQUAL, base_pb2.Filters.OPERATOR_NOT_EQUAL),
        (_Operator.LESS_THAN, base_pb2.Filters.OPERATOR_LESS_THAN),
        (_Operator.LESS_THAN_EQUAL, base_pb2.Filters.OPERATOR_LESS_THAN_EQUAL),
        (_Operator.GREATER_THAN, base_pb2.Filters.OPERATOR_GREATER_THAN),
        (_Operator.GREATER_THAN_EQUAL, base_pb2.Filters.OPERATOR_GREATER_THAN_EQUAL),
        (_Operator.LIKE, base_pb2.Filters.OPERATOR_LIKE),
        (_Operator.IS_NULL, base_pb2.Filters.OPERATOR_IS_NULL),
        (_Operator.CONTAINS_ANY, base_pb2.Filters.OPERATOR_CONTAINS_ANY),
        (_Operator.CONTAINS_ALL, base_pb2.Filters.OPERATOR_CONTAINS_ALL),
        (_Operator.CONTAINS_NONE, base_pb2.Filters.OPERATOR_CONTAINS_NONE),
        (_Operator.WITHIN_GEO_RANGE, base_pb2.Filters.OPERATOR_WITHIN_GEO_RANGE),
        (_Operator.AND, base_pb2.Filters.OPERATOR_AND),
        (_Operator.OR, base_pb2.Filters.OPERATOR_OR),
        (_Operator.NOT, base_pb2.Filters.OPERATOR_NOT),
    ],
)
def test_operator_to_grpc(operator: _Operator, want: base_pb2.Filters.Operator) -> None:
    assert operator._to_grpc() == want, "wrong pb operator"



from pathlib import Path
from typing import Callable, Union

import pytest

from weaviate.collections.classes.generative import (
    GenerativeConfig,
    GenerativeParameters,
    _GenerativeConfigRuntimeOptions,
    _GroupedTask,
    _SinglePrompt,
)
from weaviate.proto.v1 import base_pb2
from weaviate.proto.v1 import generative_pb2
from weaviate.types import BLOB_INPUT

LOGO = "test/collection/weaviate-logo.png"
LOGO_ENCODED = " "


@pytest.mark.parametrize(
    "provider",
    [
        lambda image: GenerativeParameters.grouped_task("whatever", images=[image]),
        lambda image: GenerativeParameters.single_prompt("whatever", images=[image]),
        lambda image: GenerativeParameters.grouped_task("whatever", images=image),
        lambda image: GenerativeParameters.single_prompt("whatever", images=image),
    ],
)
def test_generative_parameters_images_parsing(
    provider: Union[
        Callable[[BLOB_INPUT], _GroupedTask],
        Callable[[BLOB_INPUT], _SinglePrompt],
    ],
) -> None:
    # tests image as a path string
    p = provider(LOGO)
    assert p.images is not None
    assert list(p.images) == [LOGO_ENCODED]

    # tests image as a path object
    p = provider(Path(LOGO))
    assert p.images is not None
    assert list(p.images) == [LOGO_ENCODED]

    # tests image as an open file
    with open(LOGO, "rb") as f:
        p = provider(f)
        assert p.images is not None
        assert list(p.images) == [LOGO_ENCODED]


@pytest.mark.parametrize(
    "actual,expected",
    [
        (
            GenerativeConfig.anthropic(
                base_url="http://localhost:8080",
                model="text-to-image",
                max_tokens=100,
                stop_sequences=["\n"],
                temperature=0.5,
                top_k=50,
                top_p=0.9,
            )._to_grpc(
                _GenerativeConfigRuntimeOptions(
                    return_metadata=True, images=[LOGO_ENCODED], image_properties=["image"]
                )
            ),
            generative_pb2.GenerativeProvider(
                return_metadata=True,
                anthropic=generative_pb2.GenerativeAnthropic(
                    base_url="http://localhost:8080",
                    model="text-to-image",
                    max_tokens=100,
                    stop_sequences=base_pb2.TextArray(values=["\n"]),
                    temperature=0.5,
                    top_k=50,
                    top_p=0.9,
                    images=base_pb2.TextArray(values=[LOGO_ENCODED]),
                    image_properties=base_pb2.TextArray(values=["image"]),
                ),
            ),
        ),
        (
            GenerativeConfig.anyscale(
                base_url="http://localhost:8080",
                model="text-to-image",
                temperature=0.5,
            )._to_grpc(_GenerativeConfigRuntimeOptions(return_metadata=True)),
            generative_pb2.GenerativeProvider(
                return_metadata=True,
                anyscale=generative_pb2.GenerativeAnyscale(
                    base_url="http://localhost:8080",
                    model="text-to-image",
                    temperature=0.5,
                ),
            ),
        ),
        (
            GenerativeConfig.aws(
                endpoint="http://localhost:8080",
                model="text-to-image",
                region="us-west-2",
                service="sagemaker",
                target_model="arn:aws:sagemaker:us-west-2:123456789012:model/text-to-image",
                target_variant="variant-1",
                temperature=0.5,
            )._to_grpc(
                _GenerativeConfigRuntimeOptions(
                    return_metadata=True, images=[LOGO_ENCODED], image_properties=["image"]
                )
            ),
            generative_pb2.GenerativeProvider(
                return_metadata=True,
                aws=generative_pb2.GenerativeAWS(
                    endpoint="http://localhost:8080",
                    model="text-to-image",
                    region="us-west-2",
                    service="sagemaker",
                    target_model="arn:aws:sagemaker:us-west-2:123456789012:model/text-to-image",
                    target_variant="variant-1",
                    temperature=0.5,
                    images=base_pb2.TextArray(values=[LOGO_ENCODED]),
                    image_properties=base_pb2.TextArray(values=["image"]),
                ),
            ),
        ),
        (
            GenerativeConfig.cohere(
                base_url="http://localhost:8080",
                k=5,
                max_tokens=100,
                model="text-to-image",
                p=0.9,
                presence_penalty=0.5,
                stop_sequences=["\n"],
                temperature=0.5,
            )._to_grpc(_GenerativeConfigRuntimeOptions(return_metadata=True)),
            generative_pb2.GenerativeProvider(
                return_metadata=True,
                cohere=generative_pb2.GenerativeCohere(
                    base_url="http://localhost:8080",
                    k=5,
                    max_tokens=100,
                    model="text-to-image",
                    p=0.9,
                    presence_penalty=0.5,
                    stop_sequences=base_pb2.TextArray(values=["\n"]),
                    temperature=0.5,
                ),
            ),
        ),
        (
            GenerativeConfig.databricks(
                endpoint="http://localhost:8080",
                frequency_penalty=0.5,
                log_probs=True,
                max_tokens=100,
                model="text-to-image",
                n=5,
                presence_penalty=0.5,
                stop=["\n"],
                temperature=0.5,
                top_log_probs=5,
                top_p=0.9,
            )._to_grpc(_GenerativeConfigRuntimeOptions(return_metadata=True)),
            generative_pb2.GenerativeProvider(
                return_metadata=True,
                databricks=generative_pb2.GenerativeDatabricks(
                    endpoint="http://localhost:8080",
                    frequency_penalty=0.5,
                    log_probs=True,
                    max_tokens=100,
                    model="text-to-image",
                    n=5,
                    presence_penalty=0.5,
                    stop=base_pb2.TextArray(values=["\n"]),
                    temperature=0.5,
                    top_log_probs=5,
                    top_p=0.9,
                ),
            ),
        ),
        (
            GenerativeConfig.dummy()._to_grpc(
                _GenerativeConfigRuntimeOptions(return_metadata=True)
            ),
            generative_pb2.GenerativeProvider(
                return_metadata=True, dummy=generative_pb2.GenerativeDummy()
            ),
        ),
        (
            GenerativeConfig.friendliai(
                base_url="http://localhost:8080",
                max_tokens=100,
                model="text-to-image",
                n=5,
                temperature=0.5,
                top_p=0.9,
            )._to_grpc(_GenerativeConfigRuntimeOptions(return_metadata=True)),
            generative_pb2.GenerativeProvider(
                return_metadata=True,
                friendliai=generative_pb2.GenerativeFriendliAI(
                    base_url="http://localhost:8080",
                    max_tokens=100,
                    model="text-to-image",
                    n=5,
                    temperature=0.5,
                    top_p=0.9,
                ),
            ),
        ),
        (
            GenerativeConfig.google(
                api_endpoint="http://localhost:8080",
                endpoint_id="12345678901234567890123456789012",
                frequency_penalty=0.5,
                max_tokens=100,
                model="text-to-image",
                presence_penalty=0.5,
                project_id="my-project",
                region="us-west1",
                stop_sequences=["\n"],
                temperature=0.5,
                top_k=50,
                top_p=0.9,
            )._to_grpc(
                _GenerativeConfigRuntimeOptions(
                    return_metadata=True, images=[LOGO_ENCODED], image_properties=["image"]
                )
            ),
            generative_pb2.GenerativeProvider(
                return_metadata=True,
                google=generative_pb2.GenerativeGoogle(
                    api_endpoint="localhost:8080",
                    endpoint_id="12345678901234567890123456789012",
                    frequency_penalty=0.5,
                    max_tokens=100,
                    model="text-to-image",
                    presence_penalty=0.5,
                    project_id="my-project",
                    region="us-west1",
                    stop_sequences=base_pb2.TextArray(values=["\n"]),
                    temperature=0.5,
                    top_k=50,
                    top_p=0.9,
                    images=base_pb2.TextArray(values=[LOGO_ENCODED]),
                    image_properties=base_pb2.TextArray(values=["image"]),
                ),
            ),
        ),
        (
            GenerativeConfig.mistral(
                base_url="http://localhost:8080",
                max_tokens=100,
                model="text-to-image",
                temperature=0.5,
                top_p=0.9,
            )._to_grpc(_GenerativeConfigRuntimeOptions(return_metadata=True)),
            generative_pb2.GenerativeProvider(
                return_metadata=True,
                mistral=generative_pb2.GenerativeMistral(
                    base_url="http://localhost:8080",
                    max_tokens=100,
                    model="text-to-image",
                    temperature=0.5,
                    top_p=0.9,
                ),
            ),
        ),
        (
            GenerativeConfig.nvidia(
                base_url="http://localhost:8080",
                max_tokens=100,
                model="text-to-image",
                temperature=0.5,
                top_p=0.9,
            )._to_grpc(_GenerativeConfigRuntimeOptions(return_metadata=True)),
            generative_pb2.GenerativeProvider(
                return_metadata=True,
                nvidia=generative_pb2.GenerativeNvidia(
                    base_url="http://localhost:8080",
                    max_tokens=100,
                    model="text-to-image",
                    temperature=0.5,
                    top_p=0.9,
                ),
            ),
        ),
        (
            GenerativeConfig.ollama(
                api_endpoint="http://localhost:8080",
                model="text-to-image",
                temperature=0.5,
            )._to_grpc(
                _GenerativeConfigRuntimeOptions(
                    return_metadata=True, images=[LOGO_ENCODED], image_properties=["image"]
                )
            ),
            generative_pb2.GenerativeProvider(
                return_metadata=True,
                ollama=generative_pb2.GenerativeOllama(
                    api_endpoint="http://localhost:8080",
                    model="text-to-image",
                    temperature=0.5,
                    images=base_pb2.TextArray(values=[LOGO_ENCODED]),
                    image_properties=base_pb2.TextArray(values=["image"]),
                ),
            ),
        ),
        (
            GenerativeConfig.openai(
                api_version="2020-05-03",
                base_url="http://localhost:8080",
                deployment_id="12345678901234567890123456789012",
                frequency_penalty=0.5,
                max_tokens=100,
                model="text-to-image",
                presence_penalty=0.5,
                resource_name="projects/my-project/locations/us-west1/endpoints/12345678901234567890123456789012",
                stop=["\n"],
                temperature=0.5,
                top_p=50,
                reasoning_effort="high",
                verbosity="low",
            )._to_grpc(
                _GenerativeConfigRuntimeOptions(
                    return_metadata=True, images=[LOGO_ENCODED], image_properties=["image"]
                )
            ),
            generative_pb2.GenerativeProvider(
                return_metadata=True,
                openai=generative_pb2.GenerativeOpenAI(
                    api_version="2020-05-03",
                    base_url="http://localhost:8080",
                    deployment_id="12345678901234567890123456789012",
                    frequency_penalty=0.5,
                    max_tokens=100,
                    model="text-to-image",
                    presence_penalty=0.5,
                    resource_name="projects/my-project/locations/us-west1/endpoints/12345678901234567890123456789012",
                    stop=base_pb2.TextArray(values=["\n"]),
                    temperature=0.5,
                    top_p=50,
                    images=base_pb2.TextArray(values=[LOGO_ENCODED]),
                    image_properties=base_pb2.TextArray(values=["image"]),
                    is_azure=False,
                    reasoning_effort=generative_pb2.GenerativeOpenAI.ReasoningEffort.REASONING_EFFORT_HIGH,
                    verbosity=generative_pb2.GenerativeOpenAI.Verbosity.VERBOSITY_LOW,
                ),
            ),
        ),
        (
            GenerativeConfig.azure_openai(
                api_version="2020-05-03",
                base_url="http://localhost:8080",
                deployment_id="12345678901234567890123456789012",
                frequency_penalty=0.5,
                max_tokens=100,
                model="text-to-image",
                presence_penalty=0.5,
                resource_name="projects/my-project/locations/us-west1/endpoints/12345678901234567890123456789012",
                stop=["\n"],
                temperature=0.5,
                top_p=50,
            )._to_grpc(
                _GenerativeConfigRuntimeOptions(
                    return_metadata=True, images=[LOGO_ENCODED], image_properties=["image"]
                )
            ),
            generative_pb2.GenerativeProvider(
                return_metadata=True,
                openai=generative_pb2.GenerativeOpenAI(
                    api_version="2020-05-03",
                    base_url="http://localhost:8080",
                    deployment_id="12345678901234567890123456789012",
                    frequency_penalty=0.5,
                    max_tokens=100,
                    model="text-to-image",
                    presence_penalty=0.5,
                    resource_name="projects/my-project/locations/us-west1/endpoints/12345678901234567890123456789012",
                    stop=base_pb2.TextArray(values=["\n"]),
                    temperature=0.5,
                    top_p=50,
                    images=base_pb2.TextArray(values=[LOGO_ENCODED]),
                    image_properties=base_pb2.TextArray(values=["image"]),
                    is_azure=True,
                ),
            ),
        ),
        (
            GenerativeConfig.xai(
                base_url="http://localhost:8080",
                max_tokens=100,
                model="text-to-image",
                temperature=0.5,
                top_p=50,
            )._to_grpc(
                _GenerativeConfigRuntimeOptions(
                    return_metadata=True, images=[LOGO_ENCODED], image_properties=["image"]
                )
            ),
            generative_pb2.GenerativeProvider(
                return_metadata=True,
                xai=generative_pb2.GenerativeXAI(
                    base_url="http://localhost:8080",
                    max_tokens=100,
                    model="text-to-image",
                    temperature=0.5,
                    top_p=50,
                    images=base_pb2.TextArray(values=[LOGO_ENCODED]),
                    image_properties=base_pb2.TextArray(values=["image"]),
                ),
            ),
        ),
    ],
)
def test_generative_provider_to_grpc(
    actual: generative_pb2.GenerativeProvider, expected: generative_pb2.GenerativeProvider
) -> None:
    assert expected == actual



from weaviate.collections.classes.tenants import (
    Tenant,
    TenantActivityStatus,
    TenantCreate,
    TenantCreateActivityStatus,
    TenantUpdate,
    TenantUpdateActivityStatus,
)
from weaviate.collections.tenants import TenantCreateInputType, TenantUpdateInputType

__all__ = [
    "Tenant",
    "TenantCreate",
    "TenantUpdate",
    "TenantActivityStatus",
    "TenantCreateActivityStatus",
    "TenantUpdateActivityStatus",
    "TenantCreateInputType",
    "TenantUpdateInputType",
]



from typing import Any, Dict, Union, cast

from pydantic import BaseModel


class _BackupLocationConfig(BaseModel):
    """The dynamic location of a backup."""

    def _to_dict(self) -> Dict[str, Any]:
        ret = cast(dict, self.model_dump(exclude_none=True))

        return ret


class _BackupLocationFilesystem(_BackupLocationConfig):
    """The dynamic location of a backup for filesystem."""

    path: str


class _BackupLocationS3(_BackupLocationConfig):
    """The dynamic location of a backup for S3."""

    path: str
    bucket: str


class _BackupLocationGCP(_BackupLocationConfig):
    """The dynamic location of a backup for GCP."""

    path: str
    bucket: str


class _BackupLocationAzure(_BackupLocationConfig):
    """The dynamic location of a backup for Azure."""

    path: str
    bucket: str


BackupLocationType = Union[
    _BackupLocationFilesystem,
    _BackupLocationS3,
    _BackupLocationGCP,
    _BackupLocationAzure,
]


class BackupLocation:
    """The dynamic path of a backup."""

    FileSystem = _BackupLocationFilesystem
    S3 = _BackupLocationS3
    GCP = _BackupLocationGCP
    Azure = _BackupLocationAzure



from typing import Any, Dict, List, Literal, Optional, Union

from pydantic import AnyHttpUrl, Field
from typing_extensions import deprecated as typing_deprecated

from weaviate.collections.classes.config_base import (
    _ConfigCreateModel,
    _ConfigUpdateModel,
    _EnumLikeStr,
)
from weaviate.collections.classes.config_vector_index import (
    VectorIndexType,
    _MultiVectorConfigCreate,
    _MultiVectorEncodingConfigCreate,
    _QuantizerConfigCreate,
    _VectorIndexConfigCreate,
    _VectorIndexConfigDynamicCreate,
    _VectorIndexConfigDynamicUpdate,
    _VectorIndexConfigFlatCreate,
    _VectorIndexConfigFlatUpdate,
    _VectorIndexConfigHNSWCreate,
    _VectorIndexConfigHNSWUpdate,
    _VectorIndexConfigUpdate,
)
from weaviate.collections.classes.config_vectorizers import (
    AWSModel,
    AWSService,
    CohereModel,
    CohereMultimodalModel,
    CohereTruncation,
    JinaModel,
    JinaMultimodalModel,
    Multi2VecField,
    OpenAIModel,
    OpenAIType,
    Vectorizers,
    VoyageModel,
    VoyageMultimodalModel,
    WeaviateModel,
    _Img2VecNeuralConfig,
    _map_multi2vec_fields,
    _Multi2MultiVecJinaConfig,
    _Multi2VecAWSConfig,
    _Multi2VecBindConfig,
    _Multi2VecClipConfig,
    _Multi2VecCohereConfig,
    _Multi2VecGoogleConfig,
    _Multi2VecJinaConfig,
    _Multi2VecNvidiaConfig,
    _Multi2VecVoyageaiConfig,
    _Ref2VecCentroidConfig,
    _Text2ColbertJinaAIConfig,
    _Text2VecAWSConfig,
    _Text2VecAzureOpenAIConfig,
    _Text2VecCohereConfig,
    _Text2VecContextionaryConfig,
    _Text2VecDatabricksConfig,
    _Text2VecGoogleConfig,
    _Text2VecGPT4AllConfig,
    _Text2VecHuggingFaceConfig,
    _Text2VecJinaConfig,
    _Text2VecMistralConfig,
    _Text2VecModel2VecConfig,
    _Text2VecMorphConfig,
    _Text2VecNvidiaConfig,
    _Text2VecOllamaConfig,
    _Text2VecOpenAIConfig,
    _Text2VecTransformersConfig,
    _Text2VecVoyageConfig,
    _Text2VecWeaviateConfig,
    _VectorizerConfigCreate,
    _VectorizerCustomConfig,
)


class _VectorConfigCreate(_ConfigCreateModel):
    name: Optional[str]
    properties: Optional[List[str]] = Field(default=None, min_length=1, alias="source_properties")
    vectorizer: _VectorizerConfigCreate
    vectorIndexType: VectorIndexType = Field(default=VectorIndexType.HNSW, exclude=True)
    vectorIndexConfig: Optional[_VectorIndexConfigCreate] = Field(
        default=None, alias="vector_index_config"
    )

    def _to_dict(self) -> Dict[str, Any]:
        ret_dict: Dict[str, Any] = self.__parse_vectorizer()
        if self.vectorIndexConfig is not None:
            ret_dict["vectorIndexType"] = self.vectorIndexConfig.vector_index_type().value
            ret_dict["vectorIndexConfig"] = self.vectorIndexConfig._to_dict()
        else:
            ret_dict["vectorIndexType"] = self.vectorIndexType.value
        return ret_dict

    def __parse_vectorizer(self) -> Dict[str, Any]:
        vectorizer_options = self.vectorizer._to_dict()
        if self.properties is not None:
            vectorizer_options["properties"] = self.properties
        return {"vectorizer": {self.vectorizer.vectorizer.value: vectorizer_options}}


class _VectorConfigUpdate(_ConfigUpdateModel):
    name: str
    vectorIndexConfig: _VectorIndexConfigUpdate = Field(..., alias="vector_index_config")


class _IndexWrappers:
    @staticmethod
    def __hnsw(
        *,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        multivector: Optional[_MultiVectorConfigCreate] = None,
    ) -> _VectorIndexConfigHNSWCreate:
        return _VectorIndexConfigHNSWCreate(
            cleanupIntervalSeconds=None,
            distance=None,
            dynamicEfMin=None,
            dynamicEfMax=None,
            dynamicEfFactor=None,
            efConstruction=None,
            ef=None,
            filterStrategy=None,
            flatSearchCutoff=None,
            maxConnections=None,
            vectorCacheMaxObjects=None,
            quantizer=quantizer,
            multivector=multivector,
        )

    @staticmethod
    def __flat(*, quantizer: Optional[_QuantizerConfigCreate]) -> _VectorIndexConfigFlatCreate:
        return _VectorIndexConfigFlatCreate(
            distance=None,
            vectorCacheMaxObjects=None,
            quantizer=quantizer,
            multivector=None,
        )

    @staticmethod
    def single(
        vector_index_config: Optional[_VectorIndexConfigCreate],
        quantizer: Optional[_QuantizerConfigCreate],
    ) -> Optional[_VectorIndexConfigCreate]:
        if quantizer is not None:
            if vector_index_config is None:
                vector_index_config = _IndexWrappers.__hnsw(quantizer=quantizer)
            else:
                if isinstance(vector_index_config, _VectorIndexConfigDynamicCreate):
                    if vector_index_config.hnsw is None:
                        vector_index_config.hnsw = _IndexWrappers.__hnsw(quantizer=quantizer)
                    else:
                        vector_index_config.hnsw.quantizer = quantizer
                    if vector_index_config.flat is None:
                        vector_index_config.flat = _IndexWrappers.__flat(quantizer=quantizer)
                    else:
                        vector_index_config.flat.quantizer = quantizer
                else:
                    vector_index_config.quantizer = quantizer
        return vector_index_config

    @staticmethod
    def multi(
        vector_index_config: Optional[_VectorIndexConfigCreate],
        quantizer: Optional[_QuantizerConfigCreate],
        multi_vector_config: Optional[_MultiVectorConfigCreate],
        encoding: Optional[_MultiVectorEncodingConfigCreate],
    ) -> Optional[_VectorIndexConfigCreate]:
        if multi_vector_config is None:
            multi_vector_config = _MultiVectorConfigCreate(aggregation=None, encoding=None)
        if encoding is not None:
            multi_vector_config.encoding = encoding
        if vector_index_config is None:
            vector_index_config = _IndexWrappers.__hnsw(multivector=multi_vector_config)
        else:
            vector_index_config.multivector = multi_vector_config
        return _IndexWrappers.single(vector_index_config, quantizer)


class _MultiVectors:
    @staticmethod
    def self_provided(
        *,
        name: Optional[str] = None,
        encoding: Optional[_MultiVectorEncodingConfigCreate] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        multi_vector_config: Optional[_MultiVectorConfigCreate] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
    ):
        """Create a multi-vector using no vectorizer. You will need to provide the vectors yourself.

        Args:
            name: The name of the vector.
            encoding: The type of multi-vector encoding to use in the vector index. Defaults to `None`, which uses the server-defined default.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            multi_vector_config: The configuration for the multi-vector index. Use `wvc.config.Configure.VectorIndex.MultiVector` to create a multi-vector configuration. None by default
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default
        """
        return _VectorConfigCreate(
            name=name,
            vectorizer=_VectorizerConfigCreate(vectorizer=Vectorizers.NONE),
            vector_index_config=_IndexWrappers.multi(
                vector_index_config, quantizer, multi_vector_config, encoding
            ),
        )

    @staticmethod
    def text2vec_jinaai(
        *,
        name: Optional[str] = None,
        encoding: Optional[_MultiVectorEncodingConfigCreate] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        dimensions: Optional[int] = None,
        model: Optional[str] = None,
        source_properties: Optional[List[str]] = None,
        multi_vector_config: Optional[_MultiVectorConfigCreate] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _VectorConfigCreate:
        """Create a multi-vector using the `text2colbert-jinaai` module.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/jinaai/colbert) for detailed usage.

        Args:
            name: The name of the vector.
            encoding: The type of multi-vector encoding to use in the vector index. Defaults to `None`, which uses the server-defined default.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            dimensions: Number of dimensions. Applicable to v3 OpenAI models only. Defaults to `None`, which uses the server-defined default.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            encoding: The type of multi-vector encoding to use in the vector index. Defaults to `None`, which uses the server-defined default.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            multi_vector_config: The configuration for the multi-vector index. Use `wvc.config.Configure.VectorIndex.MultiVector` to create a multi-vector configuration. None by default
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
        """
        return _VectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vector_index_config=_IndexWrappers.multi(
                vector_index_config, quantizer, multi_vector_config, encoding
            ),
            vectorizer=_Text2ColbertJinaAIConfig(
                model=model, dimensions=dimensions, vectorizeClassName=vectorize_collection_name
            ),
        )

    @staticmethod
    def multi2vec_jinaai(
        *,
        name: Optional[str] = None,
        encoding: Optional[_MultiVectorEncodingConfigCreate] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        base_url: Optional[AnyHttpUrl] = None,
        image_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        model: Optional[Union[JinaMultimodalModel, str]] = None,
        text_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        multi_vector_config: Optional[_MultiVectorConfigCreate] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
    ) -> _VectorConfigCreate:
        """Create a vector using the `multi2multivec-jinaai` module.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/jinaai/embeddings-multimodal)
        for detailed usage.

        Args:
            name: The name of the vector.
            encoding: The type of multi-vector encoding to use in the vector index. Defaults to `None`, which uses the server-defined default.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            base_url: The base URL to use where API requests should go. Defaults to `None`, which uses the server-defined default.
            image_fields: The image fields to use in vectorization.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            text_fields: The text fields to use in vectorization.
            multi_vector_config: The configuration for the multi-vector index. Use `wvc.config.Configure.VectorIndex.MultiVector` to create a multi-vector configuration. None by default
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default

        Raises:
            pydantic.ValidationError: If `model` is not a valid value from the `JinaMultimodalModel` type.
        """
        return _VectorConfigCreate(
            name=name,
            vectorizer=_Multi2MultiVecJinaConfig(
                baseURL=base_url,
                model=model,
                imageFields=_map_multi2vec_fields(image_fields),
                textFields=_map_multi2vec_fields(text_fields),
            ),
            vector_index_config=_IndexWrappers.multi(
                vector_index_config, quantizer, multi_vector_config, encoding
            ),
        )


class _Vectors:
    @staticmethod
    def self_provided(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
    ):
        """Create a vector using no vectorizer. You will need to provide the vectors yourself.

        Args:
            name: The name of the vector.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default
        """
        return _VectorConfigCreate(
            name=name,
            vectorizer=_VectorizerConfigCreate(vectorizer=Vectorizers.NONE),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    def custom(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        module_name: str,
        module_config: Optional[Dict[str, Any]] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
    ) -> _VectorConfigCreate:
        """Create a vector using a custom module that is not currently supported by the SDK.

        Args:
            name: The name of the vector.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            module_name: The name of the custom module to use.
            module_config: The configuration of the custom module to use.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default
        """
        return _VectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_VectorizerCustomConfig(
                vectorizer=_EnumLikeStr(module_name), module_config=module_config
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    def text2vec_cohere(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        base_url: Optional[AnyHttpUrl] = None,
        model: Optional[Union[CohereModel, str]] = None,
        truncate: Optional[CohereTruncation] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _VectorConfigCreate:
        """Create a vector using the `text2vec-cohere` module.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/cohere/embeddings)
        for detailed usage.

        Args:
            name: The name of the vector.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            base_url: The base URL to use where API requests should go. Defaults to `None`, which uses the server-defined default.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            truncate: The truncation strategy to use. Defaults to `None`, which uses the server-defined default.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.

        Raises:
            pydantic.ValidationError: If `model` is not a valid value from the `CohereModel` type or if `truncate` is not a valid value from the `CohereTruncation` type.
        """
        return _VectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecCohereConfig(
                baseURL=base_url,
                model=model,
                truncate=truncate,
                vectorizeClassName=vectorize_collection_name,
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    def multi2vec_cohere(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        base_url: Optional[AnyHttpUrl] = None,
        image_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        model: Optional[Union[CohereMultimodalModel, str]] = None,
        text_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        truncate: Optional[CohereTruncation] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _VectorConfigCreate:
        """Create a vector using the `multi2vec_cohere` module.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/cohere/embeddings-multimodal)
        for detailed usage.

        Args:
            name: The name of the vector.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            base_url: The base URL to use where API requests should go. Defaults to `None`, which uses the server-defined default.
            image_fields: The image fields to use in vectorization.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            text_fields: The text fields to use in vectorization.
            truncate: The truncation strategy to use. Defaults to `None`, which uses the server-defined default.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default

        Raises:
            pydantic.ValidationError: If `model` is not a valid value from the `CohereMultimodalModel` type or if `truncate` is not a valid value from the `CohereTruncation` type.
        """
        return _VectorConfigCreate(
            name=name,
            vectorizer=_Multi2VecCohereConfig(
                baseURL=base_url,
                model=model,
                truncate=truncate,
                imageFields=_map_multi2vec_fields(image_fields),
                textFields=_map_multi2vec_fields(text_fields),
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    @typing_deprecated(
        "The contextionary model is old and not recommended for use. If you are looking for a local, lightweight model try the new text2vec-model2vec module instead."
    )
    def text2vec_contextionary(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _VectorConfigCreate:
        """Create a vector using the `text2vec_contextionary` module.

        See the [documentation](https://weaviate.io/developers/weaviate/modules/retriever-vectorizer-modules/text2vec-contextionary)
        for detailed usage.

        Args:
            name: The name of the vector.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
        """
        return _VectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecContextionaryConfig(
                vectorizeClassName=vectorize_collection_name,
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    def text2vec_model2vec(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        inference_url: Optional[str] = None,
        vectorize_collection_name: bool = True,
    ) -> _VectorConfigCreate:
        """Create a vector using the `text2vec_model2vec` module.

        See the [documentation](https://docs.weaviate.io/weaviate/model-providers/model2vec)
        for detailed usage.

        Args:
            name: The name of the vector.
            inference_url: The inferenceUrl to use where API requests should go. Defaults to `None`, which uses the server-defined default.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
        """
        return _VectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecModel2VecConfig(
                vectorizeClassName=vectorize_collection_name,
                inferenceUrl=inference_url,
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    def text2vec_databricks(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        endpoint: str,
        instruction: Optional[str] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _VectorConfigCreate:
        """Create a vector using the `text2vec-databricks` module.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/databricks/embeddings)
        for detailed usage.

        Args:
            name: The name of the vector.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            endpoint: The endpoint to use.
            instruction: The instruction strategy to use. Defaults to `None`, which uses the server-defined default.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
        """
        return _VectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecDatabricksConfig(
                endpoint=endpoint,
                instruction=instruction,
                vectorizeClassName=vectorize_collection_name,
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    def text2vec_mistral(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        base_url: Optional[AnyHttpUrl] = None,
        model: Optional[str] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _VectorConfigCreate:
        """Create a vector using the `text2vec-mistral` module.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/mistral/embeddings)
        for detailed usage.

        Args:
            name: The name of the vector.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            base_url: The base URL to use where API requests should go. Defaults to `None`, which uses the server-defined default.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
        """
        return _VectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecMistralConfig(
                baseURL=base_url,
                model=model,
                vectorizeClassName=vectorize_collection_name,
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    def text2vec_morph(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        base_url: Optional[AnyHttpUrl] = None,
        model: Optional[str] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _VectorConfigCreate:
        """Create a vector using the `text2vec-morph` module.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/morph/embeddings)
        for detailed usage.

        Args:
            name: The name of the vector.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            base_url: The base URL to use where API requests should go. Defaults to `None`, which uses the server-defined default.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
        """
        return _VectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecMorphConfig(
                baseURL=base_url,
                model=model,
                vectorizeClassName=vectorize_collection_name,
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    def text2vec_ollama(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        api_endpoint: Optional[str] = None,
        model: Optional[str] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _VectorConfigCreate:
        """Create a vector using the `text2vec-ollama` module.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/ollama/embeddings)
        for detailed usage.

        Args:
            name: The name of the vector.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            api_endpoint: The base URL to use where API requests should go. Defaults to `None`, which uses the server-defined default.
                Docker users may need to specify an alias, such as `http://host.docker.internal:11434` so that the container can access the host machine.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.


        """
        return _VectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecOllamaConfig(
                apiEndpoint=api_endpoint,
                model=model,
                vectorizeClassName=vectorize_collection_name,
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    def text2vec_openai(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        base_url: Optional[AnyHttpUrl] = None,
        dimensions: Optional[int] = None,
        model: Optional[Union[OpenAIModel, str]] = None,
        model_version: Optional[str] = None,
        type_: Optional[OpenAIType] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _VectorConfigCreate:
        """Create a vector using the `text2vec-openai` module.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/openai/embeddings)
        for detailed usage.

        Args:
            name: The name of the vector.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            base_url: The base URL to use where API requests should go. Defaults to `None`, which uses the server-defined default.
            dimensions: Number of dimensions. Applicable to v3 OpenAI models only. Defaults to `None`, which uses the server-defined default.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            model_version: The model version to use. Defaults to `None`, which uses the server-defined default.
            type_: The type of model to use. Defaults to `None`, which uses the server-defined default.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.

        Raises:
            pydantic.ValidationError: If `type_` is not a valid value from the `OpenAIType` type.
        """
        return _VectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecOpenAIConfig(
                baseURL=base_url,
                model=model,
                modelVersion=model_version,
                type_=type_,
                vectorizeClassName=vectorize_collection_name,
                dimensions=dimensions,
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    def text2vec_aws(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        endpoint: Optional[str] = None,
        model: Optional[Union[AWSModel, str]],
        region: str,
        service: Union[AWSService, str] = "bedrock",
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _VectorConfigCreate:
        """Create a vector using the `text2vec-aws` module.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/aws/embeddings)
        for detailed usage.

        Args:
            name: The name of the vector.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            endpoint: The endpoint to use. Defaults to `None`, which uses the server-defined default.
            model: The model to use, REQUIRED.
            region: The AWS region to run the model from, REQUIRED.
            service: The AWS service to use. Defaults to `bedrock`.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
        """
        return _VectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecAWSConfig(
                model=model,
                endpoint=endpoint,
                region=region,
                service=service,
                vectorizeClassName=vectorize_collection_name,
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    def multi2vec_aws(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        dimensions: Optional[int] = None,
        image_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        model: Optional[str] = None,
        text_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        region: Optional[str] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
    ) -> _VectorConfigCreate:
        """Create a vector using the `multi2vec-aws` module.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/aws/embeddings)
        for detailed usage.

        Args:
            name: The name of the vector.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            dimensions: The number of dimensions to use. Defaults to `None`, which uses the server-defined default.
            image_fields: The image fields to use in vectorization.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            text_fields: The text fields to use in vectorization.
            region: The AWS region to run the model from. Defaults to `None`, which uses the server-defined defau
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default

        Raises:
            pydantic.ValidationError: If `model` is not a valid value from the `JinaMultimodalModel` type.
        """
        return _VectorConfigCreate(
            name=name,
            vectorizer=_Multi2VecAWSConfig(
                region=region,
                model=model,
                dimensions=dimensions,
                imageFields=_map_multi2vec_fields(image_fields),
                textFields=_map_multi2vec_fields(text_fields),
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    def img2vec_neural(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        image_fields: List[str],
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
    ) -> _VectorConfigCreate:
        """Create a vector using the `img2vec-neural` module.

        See the [documentation](https://weaviate.io/developers/weaviate/modules/retriever-vectorizer-modules/img2vec-neural)
        for detailed usage.

        Args:
            name: The name of the vector.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            image_fields: The image fields to use. This is a required field and must match the property fields of the collection that are defined as `DataType.BLOB`.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default

        Raises:
            pydantic.ValidationError: If `image_fields` is not a `list`.
        """
        return _VectorConfigCreate(
            name=name,
            vectorizer=_Img2VecNeuralConfig(imageFields=image_fields),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    def multi2vec_clip(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        inference_url: Optional[str] = None,
        image_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        text_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
    ) -> _VectorConfigCreate:
        """Create a vector using the `multi2vec-clip` module.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/transformers/embeddings-multimodal)
        for detailed usage.

        Args:
            name: The name of the vector.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            inference_url: The inference url to use where API requests should go. Defaults to `None`, which uses the server-defined default.
            image_fields: The image fields to use in vectorization.
            text_fields: The text fields to use in vectorization.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default

        """
        return _VectorConfigCreate(
            name=name,
            vectorizer=_Multi2VecClipConfig(
                imageFields=_map_multi2vec_fields(image_fields),
                textFields=_map_multi2vec_fields(text_fields),
                inferenceUrl=inference_url,
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    def multi2vec_google(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        dimensions: Optional[int] = None,
        image_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        location: str,
        model: Optional[str] = None,
        project_id: str,
        text_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        video_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        video_interval_seconds: Optional[int] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
    ) -> _VectorConfigCreate:
        """Create a vector using the `multi2vec-google` module.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/google/embeddings-multimodal)
        for detailed usage.

        Args:
            name: The name of the vector.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            dimensions: The number of dimensions to use. Defaults to `None`, which uses the server-defined default.
            image_fields: The image fields to use in vectorization.
            location: Where the model runs. REQUIRED.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            project_id: The project ID to use, REQUIRED.
            text_fields: The text fields to use in vectorization.
            video_fields: The video fields to use in vectorization.
            video_interval_seconds: Length of a video interval. Defaults to `None`, which uses the server-defined default.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default
        """
        return _VectorConfigCreate(
            name=name,
            vectorizer=_Multi2VecGoogleConfig(
                projectId=project_id,
                location=location,
                imageFields=_map_multi2vec_fields(image_fields),
                textFields=_map_multi2vec_fields(text_fields),
                videoFields=_map_multi2vec_fields(video_fields),
                dimensions=dimensions,
                modelId=model,
                videoIntervalSeconds=video_interval_seconds,
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    def multi2vec_bind(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        audio_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        depth_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        image_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        imu_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        text_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        thermal_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        video_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
    ) -> _VectorConfigCreate:
        """Create a vector using the `multi2vec-bind` module.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/imagebind/embeddings-multimodal)
        for detailed usage.

        Args:
            name: The name of the vector.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            audio_fields: The audio fields to use in vectorization.
            depth_fields: The depth fields to use in vectorization.
            image_fields: The image fields to use in vectorization.
            imu_fields: The IMU fields to use in vectorization.
            text_fields: The text fields to use in vectorization.
            thermal_fields: The thermal fields to use in vectorization.
            video_fields: The video fields to use in vectorization.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default
        """
        return _VectorConfigCreate(
            name=name,
            vectorizer=_Multi2VecBindConfig(
                audioFields=_map_multi2vec_fields(audio_fields),
                depthFields=_map_multi2vec_fields(depth_fields),
                imageFields=_map_multi2vec_fields(image_fields),
                IMUFields=_map_multi2vec_fields(imu_fields),
                textFields=_map_multi2vec_fields(text_fields),
                thermalFields=_map_multi2vec_fields(thermal_fields),
                videoFields=_map_multi2vec_fields(video_fields),
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    def multi2vec_voyageai(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        base_url: Optional[AnyHttpUrl] = None,
        image_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        model: Optional[Union[VoyageMultimodalModel, str]] = None,
        text_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        truncation: Optional[bool] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
    ) -> _VectorConfigCreate:
        """Create a vector using the `multi2vec-voyageai` module.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/voyageai/embeddings-multimodal)
        for detailed usage.

        Args:
            name: The name of the vector.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            base_url: The base URL to use where API requests should go. Defaults to `None`, which uses the server-defined default.
            image_fields: The image fields to use in vectorization.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            output_encoding: The output encoding to use. Defaults to `None`, which uses the server-defined default.
            text_fields: The text fields to use in vectorization.
            truncation: The truncation strategy to use. Defaults to `None`, which uses the server-defined default.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default

        Raises:
            pydantic.ValidationError: If `model` is not a valid value from the `VoyageaiMultimodalModel` type.
        """
        return _VectorConfigCreate(
            name=name,
            vectorizer=_Multi2VecVoyageaiConfig(
                baseURL=base_url,
                model=model,
                truncation=truncation,
                imageFields=_map_multi2vec_fields(image_fields),
                textFields=_map_multi2vec_fields(text_fields),
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    def multi2vec_nvidia(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        base_url: Optional[AnyHttpUrl] = None,
        image_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        model: Optional[str] = None,
        text_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        truncation: Optional[bool] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
    ) -> _VectorConfigCreate:
        """Create a vector using the `multi2vec-nvidia` module.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/nvidia/embeddings-multimodal)
        for detailed usage.

        Args:
            name: The name of the vector.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            base_url: The base URL to use where API requests should go. Defaults to `None`, which uses the server-defined default.
            image_fields: The image fields to use in vectorization.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            text_fields: The text fields to use in vectorization.
            truncation: The truncation strategy to use. Defaults to `None`, which uses the server-defined default.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default

        Raises:
            pydantic.ValidationError: If `model` is not a valid value from the `NvidiaMultimodalModel` type.
        """
        return _VectorConfigCreate(
            name=name,
            vectorizer=_Multi2VecNvidiaConfig(
                baseURL=base_url,
                model=model,
                truncation=truncation,
                imageFields=_map_multi2vec_fields(image_fields),
                textFields=_map_multi2vec_fields(text_fields),
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    def ref2vec_centroid(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        method: Literal["mean"] = "mean",
        reference_properties: List[str],
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
    ) -> _VectorConfigCreate:
        """Create a vector using the `ref2vec-centroid` module.

        See the [documentation](https://weaviate.io/developers/weaviate/modules/retriever-vectorizer-modules/text2vec-gpt4all)
        for detailed usage.

        Args:
            name: The name of the vector.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            method: The method to use. Defaults to `mean`.
            reference_properties: The reference properties to use in vectorization, REQUIRED.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default
        """
        return _VectorConfigCreate(
            name=name,
            vectorizer=_Ref2VecCentroidConfig(
                referenceProperties=reference_properties,
                method=method,
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    def text2vec_azure_openai(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        base_url: Optional[AnyHttpUrl] = None,
        deployment_id: str,
        dimensions: Optional[int] = None,
        model: Optional[str] = None,
        resource_name: str,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _VectorConfigCreate:
        """Create a vector using the `text2vec-openai` module running with Azure.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/openai-azure/embeddings)
        for detailed usage.

        Args:
            name: The name of the vector.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            base_url: The base URL to use where API requests should go. Defaults to `None`, which uses the server-defined default.
            deployment_id: The deployment ID to use, REQUIRED.
            dimensions: The dimensionality of the vectors. Defaults to `None`, which uses the server-defined default.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            resource_name: The resource name to use, REQUIRED.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
        """
        return _VectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecAzureOpenAIConfig(
                baseURL=base_url,
                dimensions=dimensions,
                model=model,
                resourceName=resource_name,
                deploymentId=deployment_id,
                vectorizeClassName=vectorize_collection_name,
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    @typing_deprecated(
        "The `text2vec-gpt4all` vectorizer is deprecated and will be removed in a future release. See the docs (https://docs.weaviate.io/weaviate/model-providers) for alternatives."
    )
    def text2vec_gpt4all(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _VectorConfigCreate:
        """Create a vector using the `text2vec-gpt4all` module.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/gpt4all/embeddings)
        for detailed usage.

        Args:
            name: The name of the vector.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
        """
        return _VectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecGPT4AllConfig(
                vectorizeClassName=vectorize_collection_name,
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    def text2vec_huggingface(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        endpoint_url: Optional[AnyHttpUrl] = None,
        model: Optional[str] = None,
        passage_model: Optional[str] = None,
        query_model: Optional[str] = None,
        wait_for_model: Optional[bool] = None,
        use_gpu: Optional[bool] = None,
        use_cache: Optional[bool] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _VectorConfigCreate:
        """Create a vector using the `text2vec-huggingface` module.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/huggingface/embeddings)
        for detailed usage.

        Args:
            name: The name of the vector.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            endpoint_url: The endpoint URL to use. Defaults to `None`, which uses the server-defined default.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            passage_model: The passage model to use. Defaults to `None`, which uses the server-defined default.
            query_model: The query model to use. Defaults to `None`, which uses the server-defined default.
            wait_for_model: Whether to wait for the model to be loaded. Defaults to `None`, which uses the server-defined default.
            use_gpu: Whether to use the GPU. Defaults to `None`, which uses the server-defined default.
            use_cache: Whether to use the cache. Defaults to `None`, which uses the server-defined default.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.

        Raises:
            pydantic.ValidationError: If the arguments passed to the function are invalid.
                It is important to note that some of these variables are mutually exclusive.
                See the [documentation](https://weaviate.io/developers/weaviate/model-providers/huggingface/embeddings#vectorizer-parameters) for more details.
        """
        return _VectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecHuggingFaceConfig(
                model=model,
                passageModel=passage_model,
                queryModel=query_model,
                endpointURL=endpoint_url,
                waitForModel=wait_for_model,
                useGPU=use_gpu,
                useCache=use_cache,
                vectorizeClassName=vectorize_collection_name,
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    def text2vec_google(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        api_endpoint: Optional[str] = None,
        dimensions: Optional[int] = None,
        model: Optional[str] = None,
        project_id: str,
        title_property: Optional[str] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _VectorConfigCreate:
        """Create a vector using the `text2vec-google` model.

        See the [documentation]https://weaviate.io/developers/weaviate/model-providers/google/embeddings)
        for detailed usage.

        Args:
            name: The name of the vector.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            api_endpoint: The API endpoint to use without a leading scheme such as `http://`. Defaults to `None`, which uses the server-defined default.
            dimensions: The dimensionality of the vectors. Defaults to `None`, which uses the server-defined default.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            project_id: The project ID to use, REQUIRED.
            title_property: The Weaviate property name for the `gecko-002` or `gecko-003` model to use as the title.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default.
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.

        Raises:
            pydantic.ValidationError: If `api_endpoint` is not a valid URL.
        """
        return _VectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecGoogleConfig(
                projectId=project_id,
                apiEndpoint=api_endpoint,
                dimensions=dimensions,
                modelId=model,
                vectorizeClassName=vectorize_collection_name,
                titleProperty=title_property,
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    def text2vec_google_aistudio(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        dimensions: Optional[int] = None,
        model: Optional[str] = None,
        title_property: Optional[str] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _VectorConfigCreate:
        """Create a vector using the `text2vec-google` model.

        See the [documentation]https://weaviate.io/developers/weaviate/model-providers/google/embeddings)
        for detailed usage.

        Args:
            name: The name of the vector.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            dimenions: The dimensionality of the vectors. Defaults to `None`, which uses the server-defined default.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            title_property: The Weaviate property name for the `gecko-002` or `gecko-003` model to use as the title.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.

        Raises:
            pydantic.ValidationError: If `api_endpoint` is not a valid URL.
        """
        return _VectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecGoogleConfig(
                projectId=None,
                apiEndpoint="generativelanguage.googleapis.com",
                dimensions=dimensions,
                modelId=model,
                vectorizeClassName=vectorize_collection_name,
                titleProperty=title_property,
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    def text2vec_transformers(
        *,
        name: Optional[str] = None,
        dimensions: Optional[int] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        inference_url: Optional[str] = None,
        passage_inference_url: Optional[str] = None,
        pooling_strategy: Literal["masked_mean", "cls"] = "masked_mean",
        query_inference_url: Optional[str] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _VectorConfigCreate:
        """Create a vector using the `text2vec-transformers` module.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/transformers/embeddings)
        for detailed usage.

        Args:
            name: The name of the vector.
            dimensions: The number of dimensions for the generated embeddings. Defaults to `None`, which uses the server-defined default.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            inference_url: The inferenceUrl to use where API requests should go. You can use either this OR passage/query_inference_url. Defaults to `None`, which uses the server-defined default.
            passage_inference_url: The inferenceUrl to use where passage API requests should go. You can use either this and query_inference_url OR inference_url. Defaults to `None`, which uses the server-defined default.
            pooling_strategy: The pooling strategy to use. Defaults to `masked_mean`.
            query_inference_url: The inferenceUrl to use where query API requests should go. You can use either this and passage_inference_url OR inference_url. Defaults to `None`, which uses the server-defined default.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
        """
        return _VectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecTransformersConfig(
                poolingStrategy=pooling_strategy,
                dimensions=dimensions,
                vectorizeClassName=vectorize_collection_name,
                inferenceUrl=inference_url,
                passageInferenceUrl=passage_inference_url,
                queryInferenceUrl=query_inference_url,
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    def text2vec_jinaai(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        base_url: Optional[str] = None,
        dimensions: Optional[int] = None,
        model: Optional[Union[JinaModel, str]] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _VectorConfigCreate:
        """Create a vector using the `text2vec-jinaai` module.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/jinaai/embeddings) for detailed usage.

        Args:
            name: The name of the vector.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            base_url: The base URL to send the vectorization requests to. Defaults to `None`, which uses the server-defined default.
            dimensions: The number of dimensions for the generated embeddings. Defaults to `None`, which uses the server-defined default.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
        """
        return _VectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecJinaConfig(
                baseURL=base_url,
                dimensions=dimensions,
                model=model,
                vectorizeClassName=vectorize_collection_name,
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    def multi2vec_jinaai(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        base_url: Optional[AnyHttpUrl] = None,
        dimensions: Optional[int] = None,
        image_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        model: Optional[Union[JinaMultimodalModel, str]] = None,
        text_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
    ) -> _VectorConfigCreate:
        """Create a vector using the `multi2vec-jinaai` module.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/jinaai/embeddings-multimodal)
        for detailed usage.

        Args:
            name: The name of the vector.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            base_url: The base URL to use where API requests should go. Defaults to `None`, which uses the server-defined default.
            dimensions: The number of dimensions for the generated embeddings (only available for some models). Defaults to `None`, which uses the server-defined default.
            image_fields: The image fields to use in vectorization.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            text_fields: The text fields to use in vectorization.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default

        Raises:
            pydantic.ValidationError: If `model` is not a valid value from the `JinaMultimodalModel` type.
        """
        return _VectorConfigCreate(
            name=name,
            vectorizer=_Multi2VecJinaConfig(
                baseURL=base_url,
                model=model,
                dimensions=dimensions,
                imageFields=_map_multi2vec_fields(image_fields),
                textFields=_map_multi2vec_fields(text_fields),
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    def text2vec_voyageai(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        base_url: Optional[str] = None,
        model: Optional[Union[VoyageModel, str]] = None,
        truncate: Optional[bool] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _VectorConfigCreate:
        """Create a vector using the `text2vec-voyageai` module.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/voyageai/embeddings)
        for detailed usage.

        Args:
            name: The name of the vector.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            base_url: The base URL to use where API requests should go. Defaults to `None`, which uses the server-defined default.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
                See the
                [documentation](https://weaviate.io/developers/weaviate/model-providers/voyageai/embeddings#available-models) for more details.
            truncate: Whether to truncate the input texts to fit within the context length. Defaults to `None`, which uses the server-defined default.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
        """
        return _VectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecVoyageConfig(
                model=model,
                vectorizeClassName=vectorize_collection_name,
                baseURL=base_url,
                truncate=truncate,
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    def text2vec_weaviate(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        base_url: Optional[str] = None,
        dimensions: Optional[int] = None,
        model: Optional[Union[WeaviateModel, str]] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _VectorConfigCreate:
        return _VectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecWeaviateConfig(
                model=model,
                vectorizeClassName=vectorize_collection_name,
                baseURL=base_url,
                dimensions=dimensions,
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )

    @staticmethod
    def text2vec_nvidia(
        *,
        name: Optional[str] = None,
        quantizer: Optional[_QuantizerConfigCreate] = None,
        base_url: Optional[str] = None,
        model: Optional[str] = None,
        truncate: Optional[bool] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _VectorConfigCreate:
        """Create a vector using the `text2vec-nvidia` module.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/nvidia/embeddings)
        for detailed usage.

        Args:
            name: The name of the vector.
            quantizer: The quantizer to use for the vector index. If not provided, no quantization will be applied.
            base_url: The base URL to use where API requests should go. Defaults to `None`, which uses the server-defined default.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Configure.VectorIndex` to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
                See the
                [documentation](https://weaviate.io/developers/weaviate/model-providers/nvidia/embeddings#available-models) for more details.

            truncate: Whether to truncate the input texts to fit within the context length. Defaults to `None`, which uses the server-defined default.
        """
        return _VectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecNvidiaConfig(
                model=model,
                vectorizeClassName=vectorize_collection_name,
                baseURL=base_url,
                truncate=truncate,
            ),
            vector_index_config=_IndexWrappers.single(vector_index_config, quantizer),
        )


class _VectorsUpdate:
    @staticmethod
    def update(
        *,
        name: Optional[str] = None,
        vector_index_config: Union[
            _VectorIndexConfigHNSWUpdate,
            _VectorIndexConfigFlatUpdate,
            _VectorIndexConfigDynamicUpdate,
        ],
    ) -> _VectorConfigUpdate:
        """Update the vector index configuration of a vector.

        This is the only update operation allowed currently. If you wish to change the vectorization configuration itself, you will have to
        recreate the collection with the new configuration.

        Args:
            name: The name of the vector.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Reconfigure.VectorIndex` to create a vector index configuration. `None` by default
        """
        return _VectorConfigUpdate(
            name=name or "default",
            vector_index_config=vector_index_config,
        )



from typing import Literal, Optional, Union, overload

from weaviate.collections.classes.aggregate import (
    AggregateGroupByReturn,
    AggregateReturn,
    GroupByAggregate,
    PropertiesMetrics,
)
from weaviate.collections.classes.filters import _Filters
from weaviate.connect.v4 import ConnectionSync
from weaviate.types import NUMBER, UUID

from .executor import _NearObjectExecutor

class _NearObject(_NearObjectExecutor[ConnectionSync]):
    @overload
    def near_object(
        self,
        near_object: UUID,
        *,
        certainty: Optional[NUMBER] = None,
        distance: Optional[NUMBER] = None,
        object_limit: Optional[int] = None,
        filters: Optional[_Filters] = None,
        group_by: Literal[None] = None,
        target_vector: Optional[str] = None,
        total_count: bool = True,
        return_metrics: Optional[PropertiesMetrics] = None,
    ) -> AggregateReturn: ...
    @overload
    def near_object(
        self,
        near_object: UUID,
        *,
        certainty: Optional[NUMBER] = None,
        distance: Optional[NUMBER] = None,
        object_limit: Optional[int] = None,
        filters: Optional[_Filters] = None,
        group_by: Union[str, GroupByAggregate],
        target_vector: Optional[str] = None,
        total_count: bool = True,
        return_metrics: Optional[PropertiesMetrics] = None,
    ) -> AggregateGroupByReturn: ...
    @overload
    def near_object(
        self,
        near_object: UUID,
        *,
        certainty: Optional[NUMBER] = None,
        distance: Optional[NUMBER] = None,
        object_limit: Optional[int] = None,
        filters: Optional[_Filters] = None,
        group_by: Optional[Union[str, GroupByAggregate]] = None,
        target_vector: Optional[str] = None,
        total_count: bool = True,
        return_metrics: Optional[PropertiesMetrics] = None,
    ) -> Union[AggregateReturn, AggregateGroupByReturn]: ...



from typing import Generic

from weaviate.collections.classes.types import Properties, References
from weaviate.collections.queries.near_media.generate.executor import (
    _NearMediaGenerateExecutor,
)
from weaviate.connect import executor
from weaviate.connect.v4 import ConnectionSync


@executor.wrap("sync")
class _NearMediaGenerate(
    Generic[Properties, References],
    _NearMediaGenerateExecutor[ConnectionSync, Properties, References],
):
    pass



from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class CompressionType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    COMPRESSION_TYPE_UNSPECIFIED: _ClassVar[CompressionType]
    COMPRESSION_TYPE_GZIP: _ClassVar[CompressionType]
    COMPRESSION_TYPE_ZLIB: _ClassVar[CompressionType]
    COMPRESSION_TYPE_DEFLATE: _ClassVar[CompressionType]
COMPRESSION_TYPE_UNSPECIFIED: CompressionType
COMPRESSION_TYPE_GZIP: CompressionType
COMPRESSION_TYPE_ZLIB: CompressionType
COMPRESSION_TYPE_DEFLATE: CompressionType

class PauseFileActivityRequest(_message.Message):
    __slots__ = ("index_name", "shard_name", "schema_version")
    INDEX_NAME_FIELD_NUMBER: _ClassVar[int]
    SHARD_NAME_FIELD_NUMBER: _ClassVar[int]
    SCHEMA_VERSION_FIELD_NUMBER: _ClassVar[int]
    index_name: str
    shard_name: str
    schema_version: int
    def __init__(self, index_name: _Optional[str] = ..., shard_name: _Optional[str] = ..., schema_version: _Optional[int] = ...) -> None: ...

class PauseFileActivityResponse(_message.Message):
    __slots__ = ("index_name", "shard_name")
    INDEX_NAME_FIELD_NUMBER: _ClassVar[int]
    SHARD_NAME_FIELD_NUMBER: _ClassVar[int]
    index_name: str
    shard_name: str
    def __init__(self, index_name: _Optional[str] = ..., shard_name: _Optional[str] = ...) -> None: ...

class ResumeFileActivityRequest(_message.Message):
    __slots__ = ("index_name", "shard_name")
    INDEX_NAME_FIELD_NUMBER: _ClassVar[int]
    SHARD_NAME_FIELD_NUMBER: _ClassVar[int]
    index_name: str
    shard_name: str
    def __init__(self, index_name: _Optional[str] = ..., shard_name: _Optional[str] = ...) -> None: ...

class ResumeFileActivityResponse(_message.Message):
    __slots__ = ("index_name", "shard_name")
    INDEX_NAME_FIELD_NUMBER: _ClassVar[int]
    SHARD_NAME_FIELD_NUMBER: _ClassVar[int]
    index_name: str
    shard_name: str
    def __init__(self, index_name: _Optional[str] = ..., shard_name: _Optional[str] = ...) -> None: ...

class ListFilesRequest(_message.Message):
    __slots__ = ("index_name", "shard_name")
    INDEX_NAME_FIELD_NUMBER: _ClassVar[int]
    SHARD_NAME_FIELD_NUMBER: _ClassVar[int]
    index_name: str
    shard_name: str
    def __init__(self, index_name: _Optional[str] = ..., shard_name: _Optional[str] = ...) -> None: ...

class ListFilesResponse(_message.Message):
    __slots__ = ("index_name", "shard_name", "file_names")
    INDEX_NAME_FIELD_NUMBER: _ClassVar[int]
    SHARD_NAME_FIELD_NUMBER: _ClassVar[int]
    FILE_NAMES_FIELD_NUMBER: _ClassVar[int]
    index_name: str
    shard_name: str
    file_names: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, index_name: _Optional[str] = ..., shard_name: _Optional[str] = ..., file_names: _Optional[_Iterable[str]] = ...) -> None: ...

class GetFileMetadataRequest(_message.Message):
    __slots__ = ("index_name", "shard_name", "file_name")
    INDEX_NAME_FIELD_NUMBER: _ClassVar[int]
    SHARD_NAME_FIELD_NUMBER: _ClassVar[int]
    FILE_NAME_FIELD_NUMBER: _ClassVar[int]
    index_name: str
    shard_name: str
    file_name: str
    def __init__(self, index_name: _Optional[str] = ..., shard_name: _Optional[str] = ..., file_name: _Optional[str] = ...) -> None: ...

class FileMetadata(_message.Message):
    __slots__ = ("index_name", "shard_name", "file_name", "size", "crc32")
    INDEX_NAME_FIELD_NUMBER: _ClassVar[int]
    SHARD_NAME_FIELD_NUMBER: _ClassVar[int]
    FILE_NAME_FIELD_NUMBER: _ClassVar[int]
    SIZE_FIELD_NUMBER: _ClassVar[int]
    CRC32_FIELD_NUMBER: _ClassVar[int]
    index_name: str
    shard_name: str
    file_name: str
    size: int
    crc32: int
    def __init__(self, index_name: _Optional[str] = ..., shard_name: _Optional[str] = ..., file_name: _Optional[str] = ..., size: _Optional[int] = ..., crc32: _Optional[int] = ...) -> None: ...

class GetFileRequest(_message.Message):
    __slots__ = ("index_name", "shard_name", "file_name", "compression")
    INDEX_NAME_FIELD_NUMBER: _ClassVar[int]
    SHARD_NAME_FIELD_NUMBER: _ClassVar[int]
    FILE_NAME_FIELD_NUMBER: _ClassVar[int]
    COMPRESSION_FIELD_NUMBER: _ClassVar[int]
    index_name: str
    shard_name: str
    file_name: str
    compression: CompressionType
    def __init__(self, index_name: _Optional[str] = ..., shard_name: _Optional[str] = ..., file_name: _Optional[str] = ..., compression: _Optional[_Union[CompressionType, str]] = ...) -> None: ...

class FileChunk(_message.Message):
    __slots__ = ("offset", "data", "eof")
    OFFSET_FIELD_NUMBER: _ClassVar[int]
    DATA_FIELD_NUMBER: _ClassVar[int]
    EOF_FIELD_NUMBER: _ClassVar[int]
    offset: int
    data: bytes
    eof: bool
    def __init__(self, offset: _Optional[int] = ..., data: _Optional[bytes] = ..., eof: bool = ...) -> None: ...



import datetime
import uuid
from typing import Iterable

import pytest

import weaviate.classes as wvc
from weaviate.collections.classes.config import DataType, Property
from weaviate.collections.classes.data import DataObject
from weaviate.types import UUID

from .conftest import AsyncCollectionFactory, AsyncOpenAICollectionFactory

UUID1 = uuid.UUID("806827e0-2b31-43ca-9269-24fa95a221f9")
UUID2 = uuid.uuid4()
UUID3 = uuid.uuid4()

DATE1 = datetime.datetime.strptime("2012-02-09", "%Y-%m-%d").replace(tzinfo=datetime.timezone.utc)


@pytest.mark.asyncio
async def test_fetch_objects(async_collection_factory: AsyncCollectionFactory) -> None:
    collection = await async_collection_factory(
        properties=[
            Property(name="name", data_type=DataType.TEXT),
        ],
        vectorizer_config=wvc.config.Configure.Vectorizer.none(),
    )
    await collection.data.insert_many(
        [
            {"name": "John Doe"},
        ]
    )

    res = await collection.query.fetch_objects()
    assert len(res.objects) == 1
    assert res.objects[0].properties["name"] == "John Doe"


@pytest.mark.asyncio
@pytest.mark.parametrize(
    "ids, expected_len, expected",
    [
        ([], 0, set()),
        ((), 0, set()),
        (
            [
                UUID3,
            ],
            1,
            {
                UUID3,
            },
        ),
        ([UUID1, UUID2], 2, {UUID1, UUID2}),
        ((UUID1, UUID3), 2, {UUID1, UUID3}),
        ((UUID1, UUID3, UUID3), 2, {UUID1, UUID3}),
    ],
)
async def test_fetch_objects_by_ids(
    async_collection_factory: AsyncCollectionFactory,
    ids: Iterable[UUID],
    expected_len: int,
    expected: set,
) -> None:
    collection = await async_collection_factory(
        properties=[
            Property(name="name", data_type=DataType.TEXT),
        ],
        vectorizer_config=wvc.config.Configure.Vectorizer.none(),
    )
    await collection.data.insert_many(
        [
            DataObject(properties={"name": "first"}, uuid=UUID1),
            DataObject(properties={"name": "second"}, uuid=UUID2),
            DataObject(properties={"name": "third"}, uuid=UUID3),
        ]
    )

    res = await collection.query.fetch_objects_by_ids(ids)
    assert len(res.objects) == expected_len
    assert {o.uuid for o in res.objects} == expected


@pytest.mark.asyncio
async def test_config_update(async_collection_factory: AsyncCollectionFactory) -> None:
    collection = await async_collection_factory(
        properties=[
            Property(name="name", data_type=DataType.TEXT),
        ],
        multi_tenancy_config=wvc.config.Configure.multi_tenancy(),
        vectorizer_config=wvc.config.Configure.Vectorizer.none(),
    )
    await collection.config.update(
        multi_tenancy_config=wvc.config.Reconfigure.multi_tenancy(
            auto_tenant_activation=True,
            auto_tenant_creation=True,
        )
    )


@pytest.mark.asyncio
async def test_config_add_property(async_collection_factory: AsyncCollectionFactory) -> None:
    collection = await async_collection_factory(
        properties=[
            Property(name="name", data_type=DataType.TEXT),
        ],
        vectorizer_config=wvc.config.Configure.Vectorizer.none(),
    )
    await collection.config.add_property(Property(name="age", data_type=DataType.INT))


@pytest.mark.asyncio
async def test_config_add_reference(async_collection_factory: AsyncCollectionFactory) -> None:
    collection = await async_collection_factory(
        properties=[
            Property(name="name", data_type=DataType.TEXT),
        ],
        vectorizer_config=wvc.config.Configure.Vectorizer.none(),
    )
    await collection.config.add_reference(
        wvc.config.ReferenceProperty(name="test", target_collection=collection.name)
    )


@pytest.mark.asyncio
async def test_references(async_collection_factory: AsyncCollectionFactory) -> None:
    collection = await async_collection_factory(
        properties=[
            Property(name="name", data_type=DataType.TEXT),
        ],
        vectorizer_config=wvc.config.Configure.Vectorizer.none(),
    )
    await collection.config.add_reference(
        wvc.config.ReferenceProperty(name="marriage", target_collection=collection.name)
    )
    id1 = await collection.data.insert({"name": "John Doe"})
    id2 = await collection.data.insert({"name": "Jane Doe"})

    await collection.data.reference_add(id1, "marriage", id2)
    await collection.data.reference_add(id2, "marriage", id1)

    res = await collection.query.fetch_object_by_id(
        id1, return_references=wvc.query.QueryReference(link_on="marriage")
    )
    assert res.references["marriage"].objects[0].uuid == id2
    res = await collection.query.fetch_object_by_id(
        id2, return_references=wvc.query.QueryReference(link_on="marriage")
    )
    assert res.references["marriage"].objects[0].uuid == id1

    await collection.data.reference_delete(id1, "marriage", id2)
    res = await collection.query.fetch_object_by_id(
        id1, return_references=wvc.query.QueryReference(link_on="marriage")
    )
    assert len(res.references["marriage"].objects) == 0

    await collection.data.reference_replace(id2, "marriage", id2)
    res = await collection.query.fetch_object_by_id(
        id2, return_references=wvc.query.QueryReference(link_on="marriage")
    )
    assert res.references["marriage"].objects[0].uuid == id2

    await collection.data.reference_add_many(
        [wvc.data.DataReference(from_property="marriage", from_uuid=id1, to_uuid=[id1, id2])]
    )
    res = await collection.query.fetch_object_by_id(
        id1, return_references=wvc.query.QueryReference(link_on="marriage")
    )
    assert len(res.references["marriage"].objects) == 2


@pytest.mark.asyncio
async def test_aggregate(async_collection_factory: AsyncCollectionFactory) -> None:
    collection = await async_collection_factory(
        properties=[
            Property(name="name", data_type=DataType.TEXT),
        ],
        vectorizer_config=wvc.config.Configure.Vectorizer.text2vec_contextionary(
            vectorize_collection_name=False
        ),
    )
    await collection.data.insert_many(
        [
            {"name": "John Doe"},
            {"name": "Jane Doe"},
        ]
    )
    res = await collection.aggregate.over_all()
    assert res.total_count == 2

    res = await collection.aggregate.hybrid("John", alpha=0, object_limit=10)
    assert res.total_count == 1


@pytest.mark.asyncio
async def test_iterator(async_collection_factory: AsyncCollectionFactory) -> None:
    collection = await async_collection_factory(
        properties=[
            Property(name="name", data_type=DataType.TEXT),
        ],
        vectorizer_config=wvc.config.Configure.Vectorizer.none(),
    )
    await collection.data.insert_many(
        [
            {"name": "John Doe"},
            {"name": "Jane Doe"},
        ]
    )
    names = [obj.properties["name"] async for obj in collection.iterator()]
    assert "John Doe" in names
    assert "Jane Doe" in names


@pytest.mark.asyncio
async def test_delete_many(async_collection_factory: AsyncCollectionFactory) -> None:
    collection = await async_collection_factory(
        properties=[
            Property(name="name", data_type=DataType.TEXT),
        ],
        vectorizer_config=wvc.config.Configure.Vectorizer.none(),
    )
    ret = await collection.data.insert_many(
        [
            {"name": "John Doe"},
            {"name": "Jane Doe"},
        ]
    )
    await collection.data.delete_many(wvc.query.Filter.by_property("name").equal("John Doe"))
    assert (await collection.query.fetch_object_by_id(ret.uuids[0])) is None
    assert (await collection.query.fetch_object_by_id(ret.uuids[1])) is not None


@pytest.mark.asyncio
async def test_generate(async_openai_collection: AsyncOpenAICollectionFactory) -> None:
    collection = await async_openai_collection(
        vectorizer_config=wvc.config.Configure.Vectorizer.none(),
    )
    await collection.data.insert_many(
        [
            {"text": "John Doe"},
            {"text": "Jane Doe"},
        ]
    )
    res = await collection.generate.fetch_objects(
        single_prompt="Who is this? {text}", grouped_task="Who are these people?"
    )
    assert res is not None
    assert res.generated is not None
    assert len(res.objects) == 2
    for obj in res.objects:
        assert obj.generated is not None


@pytest.mark.asyncio
@pytest.mark.parametrize(
    "ids, expected_len, expected",
    [
        ([], 0, set()),
        ((), 0, set()),
        (
            [
                UUID3,
            ],
            1,
            {
                UUID3,
            },
        ),
        ([UUID1, UUID2], 2, {UUID1, UUID2}),
        ((UUID1, UUID3), 2, {UUID1, UUID3}),
        ((UUID1, UUID3, UUID3), 2, {UUID1, UUID3}),
    ],
)
async def test_generate_by_ids(
    async_openai_collection: AsyncOpenAICollectionFactory,
    ids: Iterable[UUID],
    expected_len: int,
    expected: set,
) -> None:
    collection = await async_openai_collection(
        vectorizer_config=wvc.config.Configure.Vectorizer.none(),
    )
    await collection.data.insert_many(
        [
            DataObject(properties={"text": "John Doe"}, uuid=UUID1),
            DataObject(properties={"text": "Jane Doe"}, uuid=UUID2),
            DataObject(properties={"text": "J. Doe"}, uuid=UUID3),
        ]
    )
    res = await collection.generate.fetch_objects_by_ids(
        ids,
        single_prompt="Who is this? {text}",
        grouped_task="Who are these people?",
    )
    assert res is not None
    assert res.generated is not None
    assert len(res.objects) == expected_len
    assert {o.uuid for o in res.objects} == expected
    for obj in res.objects:
        assert obj.generated is not None



from weaviate.collections.grpc.shared import _ByteOps


def test_decode_float32s():
    assert _ByteOps.decode_float32s(b"") == []
    assert _ByteOps.decode_float32s(b"\x00\x00\x80?\x00\x00\x00@\x00\x00\x00\x00") == [
        1.0,
        2.0,
        0.0,
    ]


def test_decode_float64s():
    assert _ByteOps.decode_float64s(b"") == []
    assert _ByteOps.decode_float64s(
        b"\x00\x00\x00\x00\x00\x00\xf0?\x00\x00\x00\x00\x00\x00\x00@\x00\x00\x00\x00\x00\x00\x00\x00"
    ) == [1.0, 2.0, 0.0]


def test_decode_int64s():
    assert _ByteOps.decode_int64s(b"") == []
    assert _ByteOps.decode_int64s(
        b"\x01\x00\x00\x00\x00\x00\x00\x00\x02\x00\x00\x00\x00\x00\x00\x00"
    ) == [1, 2]



from weaviate.collections.batch.collection import (
    BatchCollection,
    CollectionBatchingContextManager,
)
from weaviate.collections.collection import Collection, CollectionAsync

__all__ = [
    "BatchCollection",
    "Collection",
    "CollectionAsync",
    "CollectionBatchingContextManager",
]
